#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import main

import os
import pickle
import exceptions

class BookmarkException (exceptions.Exception):
	def __init__ (self, args=None):
		self.args = args

class Bookmark:

	def __init__ (self):
		config_dir = main.getApp ().getConfigDir ()
		self.book_dir = os.path.join (config_dir, "bookmarks")

		if not os.path.exists (self.book_dir):
			try:
				os.mkdir (self.book_dir)
				os.chmod (self.book_dir, 0700)
			except OSError, strerror:
				raise BookmarkException, strerror

	def getBookmarkDir (self):
		return self.book_dir

	def saveBookmark (self, file, opts):
		try:
			f = open (file, 'w')
		except Exception, strerror:
			raise BookmarkException, strerror
		pickler = pickle.Pickler (f)
		pickler.dump (opts)
		f.close ()

		# Set the bookmark permission to make sure it's private
		try:
			os.chmod (file, 0600)
		except OSError, strerror:
			raise BookmarkException, strerror

	def loadBookmark (self, file):
		try:
			f = open (file)
		except IOError, strerror:
			raise BookmarkException, strerror
		pickler = pickle.Unpickler (f)
		bookmark = pickler.load ()
		f.close ()
		return bookmark

	def renameBookmark (self, old, new):
		try:
			os.rename (old, new)
		except OSError, strerror:
			raise BookmarkException, strerror

	def removeBookmark (self, file):
		try:
			os.unlink (file)
		except OSError, strerror:
			raise BookmarkException, strerror

	def addBookmarkFolder (self, folder):
		try:
			os.mkdir (folder, 0700)
		except OSError, strerror:
			raise BookmarkException, strerror

	def renameBookmarkFolder (self, old, new):
		self.renameBookmark (old, new)

	def removeBookmarkFolder (self, folder):
		try:
			os.rmdir (folder)
		except OSError, strerror:
			raise BookmarkException, strerror
