#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import main
import bookmark
import connectwin
import messages
import util
import icons.folder
import icons.quick_connect
import icons.add_folder
import icons.add_bookmark
import icons.edit_bookmark

import os
from wxPython.wx import *

class BookmarkWindow (wxFrame):

	# Menu IDs
	idCONNECT         = wxNewId ()
	idFOLDER_ADD      = wxNewId ()
	idFOLDER_RENAME   = wxNewId ()
	idFOLDER_DELETE   = wxNewId ()
	idBOOKMARK_ADD    = wxNewId ()
	idBOOKMARK_EDIT   = wxNewId ()
	idBOOKMARK_RENAME = wxNewId ()
	idBOOKMARK_DELETE = wxNewId ()

	# Toolbar IDs
	idTOOL_CONNECT      = wxNewId ()
	idTOOL_ADDFOLDER    = wxNewId ()
	idTOOL_ADDBOOKMARK  = wxNewId ()
	idTOOL_EDITBOOKMARK = wxNewId ()

	idTREE = wxNewId ()

	# Popup Menu's
	idPOPUP_RENAMEFOLDER = wxNewId ()
	idPOPUP_DELETEFOLDER = wxNewId ()
	idPOPUP_EDITBOOKMARK = wxNewId ()
	idPOPUP_RENAMEBOOKMARK = wxNewId ()
	idPOPUP_DELETEBOOKMARK = wxNewId ()

	def __init__ (self, parent):
		wxFrame.__init__ (self, parent, -1, _("Ftpcube - Bookmarks"))
		self.SetIcon (main.getApp ().getAppIcon ())
		self.SetSize ((750, 500))
		self.headers = [ _("Description"), _("Host"), _("Username"), _("Directory") ]

		self.makeMenu ()
		self.makeToolBar ()

		# Create the tree and bookmark list
		splitter = wxSplitterWindow (self, -1, style=wxSP_3D)
		self.tree = wxTreeCtrl (splitter, self.idTREE)
		self.list = wxListCtrl (splitter, -1, style=wxLC_REPORT)
		splitter.SplitVertically (self.tree, self.list, 200)

		# Set up the tree images
		self.image_list = wxImageList (15, 17)
		self.folder_index = self.image_list.Add (icons.folder.getBitmap ())
		self.tree.SetImageList (self.image_list)

		# Set up the list columns
		for i, name in zip (range (len (self.headers)), self.headers):
			self.list.InsertColumn (i, name)
		self.list.SetColumnWidth (0, 200)
		self.list.SetColumnWidth (1, 175)
		self.list.SetColumnWidth (2, 100)
		self.list.SetColumnWidth (3, 100)

		sizer = wxBoxSizer (wxVERTICAL)
		sizer.Add (splitter, 1, wxEXPAND)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)

		self.Centre ()

		try:
			self.bookmark = bookmark.Bookmark ()
		except bookmark.BookmarkException, strerror:
			messages.displayErrorDialog (self, strerror)
		self.root = self.tree.AddRoot ("bookmarks", self.folder_index, self.folder_index)
		self.loadBookmarkTree (self.bookmark.getBookmarkDir (), self.root)

		EVT_TREE_SEL_CHANGED (self, self.idTREE, self.onTreeSelect)

		# Set the popup menu
		EVT_RIGHT_DOWN (self.tree, self.onTreeRightClick)
		EVT_RIGHT_DOWN (self.list, self.onListRightClick)

		# Register double click to connect to bookmark
		EVT_LEFT_DCLICK (self.list, self.onConnect)

		# Display the root directory
		self.displayBookmarks ()

	def makeMenu (self):
		self.menu_bar = wxMenuBar ()
		self.menu_bar.Append (self.createSiteMenu (), _("&Site"))
		self.menu_bar.Append (self.createEditMenu (), _("&Edit"))
		self.SetMenuBar (self.menu_bar)

	def createSiteMenu (self):
		site_menu = wxMenu ()
		site_menu.Append (self.idCONNECT, _("&Connect"),
		                                  _("Connect to Bookmarked Site"))

		EVT_MENU (self, self.idCONNECT, self.onConnect)

		return site_menu

	def createEditMenu (self):
		edit_menu = wxMenu ()
		edit_menu.Append (self.idFOLDER_ADD, _("Add &Folder"),
		                                     _("Add a Bookmark Folder"))
		edit_menu.Append (self.idFOLDER_RENAME, _("Rename Folder"),
		                                        _("Rename a Bookmark Folder"))
		edit_menu.Append (self.idFOLDER_DELETE, _("Delete Folder"),
		                                        _("Delete a Bookmark Folder"))

		edit_menu.AppendSeparator ()

		edit_menu.Append (self.idBOOKMARK_ADD, _("Add &Bookmark"),
		                  _("Add a New Bookmark"))
		edit_menu.Append (self.idBOOKMARK_EDIT, _("&Edit Bookmark"),
		                  _("Edit an Existing Bookmark"))
		edit_menu.Append (self.idBOOKMARK_RENAME, _("Rename Bookmark"),
		                  _("Rename Selected Bookmark"))
		edit_menu.Append (self.idBOOKMARK_DELETE, _("Delete Bookmark"),
		                  _("Delete Selected Bookmark"))

		EVT_MENU (self, self.idFOLDER_ADD, self.onFolderAdd)
		EVT_MENU (self, self.idFOLDER_RENAME, self.onFolderRename)
		EVT_MENU (self, self.idFOLDER_DELETE, self.onFolderDelete)
		EVT_MENU (self, self.idBOOKMARK_ADD, self.onBookmarkAdd)
		EVT_MENU (self, self.idBOOKMARK_EDIT, self.onBookmarkEdit)
		EVT_MENU (self, self.idBOOKMARK_RENAME, self.onBookmarkRename)
		EVT_MENU (self, self.idBOOKMARK_DELETE, self.onBookmarkDelete)

		return edit_menu

	def makeToolBar (self):
		toolbar = self.CreateToolBar ()
		toolbar.SetToolBitmapSize (wxSize (32, 32))

		bitmap = icons.quick_connect.getBitmap ()
		toolbar.AddSimpleTool (self.idTOOL_CONNECT, bitmap,
		                       shortHelpString=_("Connect"))
		util.AddLineSeparator (toolbar, 16)
		bitmap = icons.add_folder.getBitmap ()
		toolbar.AddSimpleTool (self.idTOOL_ADDFOLDER, bitmap,
		                       shortHelpString=_("Add Folder"))
		bitmap = icons.add_bookmark.getBitmap ()
		toolbar.AddSimpleTool (self.idTOOL_ADDBOOKMARK, bitmap,
		                       shortHelpString=_("Add Bookmark"))
		bitmap = icons.edit_bookmark.getBitmap ()
		toolbar.AddSimpleTool (self.idTOOL_EDITBOOKMARK, bitmap,
		                       shortHelpString=_("Edit Bookmark"))

		# Set Toolbar handlers
		EVT_TOOL (toolbar, self.idTOOL_CONNECT, self.onConnect)
		EVT_TOOL (toolbar, self.idTOOL_ADDFOLDER, self.onFolderAdd)
		EVT_TOOL (toolbar, self.idTOOL_ADDBOOKMARK, self.onBookmarkAdd)
		EVT_TOOL (toolbar, self.idTOOL_EDITBOOKMARK, self.onBookmarkEdit)

		toolbar.Realize ()

	def onTreeRightClick (self, event):
		menu = self.makeTreePopupMenu ()
		self.tree.PopupMenu (menu, wxPoint (event.m_x, event.m_y))

	def makeTreePopupMenu (self):
		menu = wxMenu ()
		menu.Append (self.idPOPUP_RENAMEFOLDER, _("Rename Folder"))
		menu.Append (self.idPOPUP_DELETEFOLDER, _("Delete Folder"))
		EVT_MENU (self, self.idPOPUP_RENAMEFOLDER, self.onFolderRename)
		EVT_MENU (self, self.idPOPUP_DELETEFOLDER, self.onFolderDelete)
		return menu

	def onListRightClick (self, event):
		menu = self.makeListPopupMenu ()
		self.list.PopupMenu (menu, wxPoint (event.m_x, event.m_y))

	def makeListPopupMenu (self):
		menu = wxMenu ()
		menu.Append (self.idPOPUP_EDITBOOKMARK, _("Edit Bookmark"))
		menu.Append (self.idPOPUP_RENAMEBOOKMARK, _("Rename Bookmark"))
		menu.Append (self.idPOPUP_DELETEBOOKMARK, _("Delete Bookmark"))
		EVT_MENU (self, self.idPOPUP_EDITBOOKMARK, self.onBookmarkEdit)
		EVT_MENU (self, self.idPOPUP_RENAMEBOOKMARK, self.onBookmarkRename)
		EVT_MENU (self, self.idPOPUP_DELETEBOOKMARK, self.onBookmarkDelete)
		return menu

	def onConnect (self, event):
		# Disconnect if necessary
		thread_mgr = main.getApp ().getThreadManager ()
		main_window = main.getApp ().getMainWindow ()
		if thread_mgr.getMainThread ():
			if not main_window.promptDisconnect ():
				return
			main_window.onDisconnect (event)

		path = self.getPathToCurrentNode ()
		selected = util.getSelected (self.list).pop (0)
		name = util.getColumnText (self.list, selected, 0)
		book_file = os.path.join (path, name)
		try:
			book_opts = self.bookmark.loadBookmark (book_file)
		except bookmark.BookmarkException, strerror:
			messages.displayErrorDialog (self,
				_("Error Loading Bookmark: %(err)s")
				%{ 'err' : strerror })
			return
		parent = self.GetParent ()
		self.Destroy ()
		connect = connectwin.ConnectionWindow (parent, book_opts)
		ret = connect.ShowModal ()
		if ret == wxID_OK:
			connect_opts = connect.getOpts ()
			if connect_opts['host'] and connect_opts['port']:
				main.getApp ().initiateConnection (connect_opts)
			else:
				messages.displayErrorDialog (main.getApp ().getMainWindow (),
					_("A hostname and port must be supplied"))

	def onFolderAdd (self, event):
		selected = self.tree.GetSelections ()
		if not selected:
			node = None
			path = self.bookmark.getBookmarkDir ()
		else:
			node = selected.pop (0)
			path = self.nodeToPath (node)

		folder = messages.displayInputDialog (self,
			 _("Ftpcube - Add Bookmark Folder"),
			_("Folder Name:"))

		if folder:
			path = os.path.join (path, folder)
			try:
				self.bookmark.addBookmarkFolder (path)
			except bookmark.BookmarkException, strerror:
				messages.displayErrorDialog (self,
					_("Error Creating Bookmark Folder: %(err)s")
					%{ 'err' : strerror})
				return
			new_node = self.tree.AppendItem (node, folder, self.folder_index,
			                                               self.folder_index)
			self.tree.Expand (node)

	def onFolderRename (self, event):
		selected = self.tree.GetSelections ()
		for node in selected:
			if node == self.root:
				continue
			path = self.nodeToPath (node)
			new_folder = messages.displayInputDialog (self,
				_("Ftpcube - Rename Folder"),
				_("Rename %(file)s to:") %{ 'file' : os.path.basename (path)})
			if new_folder:
				new_path = os.path.join (os.path.dirname (path), new_folder)
				try:
					self.bookmark.renameBookmarkFolder (path, new_path)
				except bookmark.BookmarkException, strerror:
					messages.displayErrorDialog (self,
						_("Error Renaming Bookmark Folder: %(err)s")
						%{ 'err' : strerror })
					return
				parent = self.tree.GetItemParent (node)
				self.tree.Delete (node)
				new_node = self.tree.AppendItem (parent, new_folder,
				                                 self.folder_index,
				                                 self.folder_index)
				self.loadBookmarkTree (new_path, new_node)

	def onFolderDelete (self, event):
		selected = self.tree.GetSelections ()
		for node in selected:
			if node == self.root:
				continue
			path = self.nodeToPath (node)
			try:
				self.bookmark.removeBookmarkFolder (path)
			except bookmark.BookmarkException, strerror:
				messages.displayErrorDialog (self,
					_("Error Deleting Bookmark Folder: %(err)s")
					%{ 'err' : strerror })
				return
			self.tree.Delete (node)

	def onBookmarkAdd (self, event):
		config = main.getApp ().getConfig ()
		opts = config.getOpts ()
		connect = connectwin.ConnectionWindow (self, opts, _("Ftpcube - Add Bookmark"))
		ret = connect.ShowModal ()
		if ret == wxID_OK:
			book_opts = connect.getOpts ()
			if book_opts['host']:
				path = os.path.join (self.getPathToCurrentNode (),
				                     "%s-%s" %(book_opts['host'], book_opts['port']))
				try:
					self.bookmark.saveBookmark (path, book_opts)
				except bookmark.BookmarkException, strerror:
					messages.displayErrorDialog (self,
						_("Error Adding Bookmark: %(err)s")
						%{ 'err' : strerror })
					return
				self.displayBookmarks ()

	def onBookmarkEdit (self, event):
		path = self.getPathToCurrentNode ()
		selected = util.getSelected (self.list).pop (0)
		name = util.getColumnText (self.list, selected, 0)
		book_file = os.path.join (path, name)
		try:
			opts = self.bookmark.loadBookmark (book_file)
		except bookmark.BookmarkException, strerror:
			messages.displayErrorDialog (self,
				_("Error Loading Bookmark: %(err)s") %{ 'err' : strerror })
			return
		connect = connectwin.ConnectionWindow (self, opts, _("Ftpcube - Edit Bookmark"))
		ret = connect.ShowModal ()
		if ret == wxID_OK:
			book_opts = connect.getOpts ()
			try:
				self.bookmark.saveBookmark (book_file, book_opts)
			except bookmark.BookmarkException, strerror:
				messages.displayErrorDialog (self,
					_("Error Saving Bookmark: %(err)s")
					%{ 'err' : strerror })
				return
			self.displayBookmarks ()

	def onBookmarkRename (self, event):
		path = self.getPathToCurrentNode ()
		selected = util.getSelected (self.list).pop (0)
		name = util.getColumnText (self.list, selected, 0)

		new_name = messages.displayInputDialog (self,
			_("Ftpcube - Rename Bookmark"),
			_("Rename %(name)s to:") %{ 'name' : name })

		if new_name:
			try:
				self.bookmark.renameBookmark (
					os.path.join (path, name),
					os.path.join (path, new_name)
				)
			except bookmark.BookmarkException, strerror:
				messages.displayErrorDialog (self,
					_("Error Renaming Bookmark: %(err)s")
					%{ 'err' : strerror })
				return
			self.displayBookmarks ()

	def onBookmarkDelete (self, event):
		path = self.getPathToCurrentNode ()
		selected = util.getSelected (self.list).pop (0)
		name = util.getColumnText (self.list, selected, 0)
		path = os.path.join (path, name)
		try:
			self.bookmark.removeBookmark (path)
		except bookmark.BookmarkException, strerror:
			messages.displayErrorDialog (self,
				_("Error Deleting Bookmark: %(err)s")
				%{ 'err' : strerror })
			return
		self.displayBookmarks ()

	def onTreeSelect (self, event):
		node = event.GetItem ()
		self.tree.Expand (node)
		self.displayBookmarks ()

	def loadBookmarkTree (self, path, parent):
		try:
			files = os.listdir (path)
		except OSError, strerror:
			messages.displayErrorDialog (self, _("Error reading directory: %(path)s: %(err)s")
			                                   %{ 'path' : path, 'err' : strerror})
			return

		if files:
			for file in files:
				full_path = os.path.join (path, file)
				if os.path.isdir (full_path):
					node = self.tree.AppendItem (parent, file, self.folder_index,
					                                           self.folder_index)
					self.loadBookmarkTree (full_path, node)
			self.tree.Expand (parent)

	def nodeToPath (self, node):
		path = [ ]
		while node != self.root:
			path.insert (0, self.tree.GetItemText (node))
			node = self.tree.GetItemParent (node)
		path = os.path.join (self.bookmark.getBookmarkDir (), os.sep.join (path))
		return path

	def getPathToCurrentNode (self):
		selected = self.tree.GetSelections ()
		if not selected:
			return self.nodeToPath (self.root)
		else:
			node = selected.pop (0)
			return self.nodeToPath (node)

	def displayBookmarks (self):
		path = self.getPathToCurrentNode ()

		try:
			files = os.listdir (path)
		except OSError, strerror:
			messages.displayErrorDialog (self, _("Error reading directory: %(path)s: %(err)s")
			                                   %{ 'path' : path, 'err' : strerror })
			return

		self.list.Freeze ()
		self.list.DeleteAllItems ()

		index = 0
		for file in files:
			full_path = os.path.join (path, file)
			if os.path.isfile (full_path):
				try:
					bookmark_entry = self.bookmark.loadBookmark (full_path)
				except bookmark.BookmarkException, strerror:
					messages.displayErrorDialog (self, _("Error loading bookmark: ") + strerror)
				self.list.InsertStringItem (index, file)
				self.list.SetStringItem (index, 1, "%s:%s" %(bookmark_entry['host'], bookmark_entry['port']))
				self.list.SetStringItem (index, 2, bookmark_entry['username'])
				self.list.SetStringItem (index, 3, bookmark_entry['remotedir'])
				index = index + 1

		self.list.Thaw ()
