#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import defaults

import os
import pickle
import exceptions

class ConfigError (exceptions.RuntimeError):

	def __init__ (self, args=None):
		self.args = args

class Configuration:

	def __init__ (self, file):
		self.config_file = file
		self.config = { }

		config_dir = os.path.dirname (self.config_file)
		# Create configuration directory and files if necessary
		if not os.path.exists (config_dir):
			try:
				os.mkdir (config_dir)
				os.chmod (config_dir, 0700)
			except OSError, strerror:
				raise ConfigError (strerror)

		if not os.path.exists (self.config_file):
			self.config.update (defaults.defaults)
			self.saveConfig (self.config_file)
		else:
			self.config.update (self.loadConfig (self.config_file))

	def __getitem__ (self, item):
		try:
			return self.config[item]
		except KeyError, strerror:
			raise ConfigError (_("Invalid configuration option: %(opt)s")
			                     %{ 'opt' : strerror })

	def __setitem__ (self, item, val):
		self.config[item] = val
		return val

	def getPath (self):
		return self.config_file

	def getOpts (self):
		return self.config

	def saveConfig (self, path):
		file = open (path, 'w')
		pickler = pickle.Pickler (file)
		pickler.dump (self.config)
		file.close ()

		# Set the file mode after every save just in case
		try:
			os.chmod (path, 0600)
		except OSError, strerror:
			raise ConfigError (strerror)

	def loadConfig (self, path):
		file = open (path, 'r')
		pickler = pickle.Unpickler (file)
		config = pickler.load ()
		return config
