#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import main
import dialog
import messages

import re
from wxPython.wx import *

class ConnectionWindow (dialog.DialogWindow):

	def __init__ (self, parent, opts=None, title=None):
		if __debug__:
			print "Making connection window."

		if title is None:
			title = _("Ftpcube - Connect")
		dialog.DialogWindow.__init__ (self, parent, title)
		self.SetIcon (main.getApp ().getAppIcon ())

		panel = self.getDialogPanel ()

		notebook = wxNotebook (panel, -1)
		notebook_sizer = wxNotebookSizer (notebook)

		self.login_tab = LoginTab (notebook, opts)
		notebook.AddPage (self.login_tab, _("Login info"))
		self.advanced_tab = AdvancedTab (notebook, opts)
		notebook.AddPage (self.advanced_tab, _("Advanced"))
		self.settings_tab = SettingsTab (notebook, opts)
		notebook.AddPage (self.settings_tab, _("Settings"))

		esizer = wxBoxSizer (wxVERTICAL)
		esizer.Add (notebook_sizer, 1, wxEXPAND)
		panel.SetAutoLayout (true)
		panel.SetSizer (esizer)
		esizer.Fit (panel)
		esizer.SetSizeHints (panel)

		self.SetSizeHints (400, -1)
		self.renderDialog ()

	def getOpts (self):
		try:
			opts = { }
			opts.update (self.login_tab.getOpts ())
			opts.update (self.advanced_tab.getOpts ())
			opts.update (self.settings_tab.getOpts ())
			return opts
		except Exception, strerror:
			return None

class LoginTab (wxPanel):

	idUSER     = wxNewId ()
	idPASS     = wxNewId ()
	idPERSONAL = wxNewId ()
	idANON     = wxNewId ()
	idRADIO    = wxNewId ()

	ftp_url_re = re.compile (r'ftp://(([^:]+)(:([^@]+))?@)?([a-zA-Z\d\.-]+)(:(\d+))?(/.+)?')

	def __init__ (self, parent, opts=None):
		wxPanel.__init__ (self, parent, -1)

		host_label = wxStaticText (self, -1, _("Host or URL"))
		port_label = wxStaticText (self, -1, _("Port"))
		lsizer1 = wxGridSizer (1, 2)
		lsizer1.AddMany ([
			(host_label, 0, wxEXPAND),
			(port_label, 0, wxEXPAND)
		])
		lsizer1.SetHGap (20)

		self.host_entry = wxTextCtrl (self, -1)
		if opts and opts['host']:
			self.host_entry.SetValue (opts['host'])
		self.port_entry = wxTextCtrl (self, -1)
		if opts and opts['port']:
			self.port_entry.SetValue (str (opts['port']))
		subsizer = wxBoxSizer (wxHORIZONTAL)
		subsizer.Add (self.port_entry, 0, wxEXPAND)
		subsizer.Add (5, 5, 1, wxEXPAND)
		subsizer.SetItemMinSize (self.port_entry, 50, -1)
		esizer1 = wxGridSizer (1, 2)
		esizer1.AddMany ([
			(self.host_entry, 0, wxEXPAND),
			(subsizer, 0, wxEXPAND)
		])
		esizer1.SetHGap (20)

		EVT_KILL_FOCUS (self.host_entry, self.onHostURL)

		user_label = wxStaticText (self, -1, _("Username"))
		pass_label = wxStaticText (self, -1, _("Password"))
		lsizer2 = wxGridSizer (1, 2)
		lsizer2.AddMany ([
			(user_label, 0, wxEXPAND),
			(pass_label, 0, wxEXPAND),
		])
		lsizer2.SetHGap (20)

		self.user_entry = wxTextCtrl (self, self.idUSER)
		if opts and opts['username']:
			self.user_entry.SetValue (opts['username'])
		self.pass_entry = wxTextCtrl (self, self.idPASS, style=wxTE_PASSWORD)
		if opts and opts['password']:
			self.pass_entry.SetValue (opts['password'])
		esizer2 = wxGridSizer (1, 2)
		esizer2.AddMany ([
			(self.user_entry, 0, wxEXPAND),
			(self.pass_entry, 0, wxEXPAND)
		])
		esizer2.SetHGap (20)

		EVT_TEXT (self, self.idUSER, self.onTextUpdate)
		EVT_TEXT (self, self.idPASS, self.onTextUpdate)

		remote_label = wxStaticText (self, -1, _("Remote Directory"))
		self.remote_entry = wxTextCtrl (self, -1)
		if opts and opts['remotedir']:
			self.remote_entry.SetValue (opts['remotedir'])

		self.personal_button = wxRadioButton (self, self.idPERSONAL, _("Personal Login"))
		self.anon_button = wxRadioButton (self, self.idANON, _("Anonymous Login"))
		if opts and opts['username'] != main.getApp ().getConfig ()['username']:
			self.personal_button.SetValue (true)
		else:
			self.onAnonClicked (None)
		bsizer = wxBoxSizer (wxHORIZONTAL)
		bsizer.Add (5, 1, 1, wxEXPAND)
		bsizer.Add (self.personal_button, 0, wxEXPAND)
		bsizer.Add (20, 1, 0, wxEXPAND)
		bsizer.Add (self.anon_button, 0, wxEXPAND)
		bsizer.Add (5, 1, 1, wxEXPAND)

		EVT_RADIOBUTTON (self, self.idPERSONAL, self.onPersonalClicked)
		EVT_RADIOBUTTON (self, self.idANON, self.onAnonClicked)

		flexsizer = wxFlexGridSizer (14, 3)
		flexsizer.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 5), (lsizer1, 0, wxEXPAND), (10, 5),
			(10, 5), (10, 5), (10, 5),
			(10, 5), (esizer1, 0, wxEXPAND), (10, 5),
			(10, 10), (10, 5), (10, 5),
			(10, 5), (lsizer2, 0, wxEXPAND), (10, 5),
			(10, 5), (10, 5), (10, 5),
			(10, 5), (esizer2, 0, wxEXPAND), (10, 5),
			(10, 5), (10, 5), (10, 5),
			(10, 5), (remote_label, 1, wxEXPAND), (10, 5),
			(10, 5), (10, 5), (10, 5),
			(10, 5), (self.remote_entry, 1, wxEXPAND), (10, 5),
			(10, 5), (10, 5), (10, 5),
			(10, 5), (bsizer, 0, wxEXPAND), (10, 5),
		])
		flexsizer.AddGrowableCol (1)

		box = wxStaticBox (self, -1, '')
		boxsizer = wxStaticBoxSizer (box, wxHORIZONTAL)
		boxsizer.Add (flexsizer, 1, wxEXPAND)

		sizer = wxFlexGridSizer (3, 3)
		sizer.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 10), (boxsizer, 1, wxEXPAND), (10, 10),
			(10, 10), (10, 10), (10, 10),
		])
		sizer.AddGrowableCol (1)
		sizer.AddGrowableRow (1)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)
		sizer.Fit (self)
		sizer.SetSizeHints (self)

	def onHostURL (self, event):
		conn = self.ftp_url_re.match (self.host_entry.GetValue ())
		config = main.getApp ().getConfig ()
		if conn:
			conninfo = conn.groups ()
			#('user:pass@','user',':pass','pass','host.com',':123','1231','/dir')
			if conninfo[1]:
				if conninfo[1] == config['host']:
					self.anon_button.SetValue (true)
				else:
					self.personal_button.SetValue (true)
				self.user_entry.SetValue (conninfo[1])
				if conninfo[3]:
					self.pass_entry.SetValue (conninfo[3])
			else:
				self.onAnonClicked (None)
			self.host_entry.SetValue (conninfo[4])
			if conninfo[6]:
				self.port_entry.SetValue (conninfo[6])
			else:
				self.port_entry.SetValue (str (config['port']))
			if conninfo[7]:
				self.remote_entry.SetValue (conninfo[7])
			else:
				self.remote_entry.SetValue ('')

	def onTextUpdate (self, event):
		self.personal_button.SetValue (true)

	def onPersonalClicked (self, event):
		self.user_entry.SetValue ('')
		self.pass_entry.SetValue ('')
		self.personal_button.SetValue (true)

	def onAnonClicked (self, event):
		config = main.getApp ().getConfig ()
		self.user_entry.SetValue (str (config['username']))
		self.pass_entry.SetValue (str (config['password']))
		self.anon_button.SetValue (true)

	def getOpts (self):
		host = self.host_entry.GetValue ()
		host = host.strip ()
		username = self.user_entry.GetValue ()
		username = username.strip ()

		return {
			'host' : host,
			'port' : int (self.port_entry.GetValue ()),
			'username' : username,
			'password' : self.pass_entry.GetValue (),
			'remotedir' : self.remote_entry.GetValue (),
		}

class AdvancedTab (wxPanel):

	idBUTTON_BROWSE   = wxNewId ()
	idRADIO_UNLIMITED = wxNewId ()
	idRADIO_ONCE      = wxNewId ()
	idRADIO_LIMIT     = wxNewId ()

	def __init__ (self, parent, opts=None):
		wxPanel.__init__ (self, parent, -1)

		label = wxStaticText (self, -1, _("Local Directory"))
		self.local_entry = wxTextCtrl (self, -1)
		if opts and opts['localdir']:
			self.local_entry.SetValue (opts['localdir'])
		self.browse_button = wxButton (self, self.idBUTTON_BROWSE,
		                               '...', size=wxSize (25, 25))

		EVT_BUTTON (self, self.idBUTTON_BROWSE, self.onBrowseClicked)

		lsizer = wxBoxSizer (wxHORIZONTAL)
		lsizer.Add (10, 1)
		lsizer.Add (label, 0, wxEXPAND)
		lsizer.Add (10, 1)
		esizer = wxBoxSizer (wxHORIZONTAL)
		esizer.Add (10, 1)
		esizer.Add (self.local_entry, 1, wxEXPAND)
		esizer.Add (20, 1)
		esizer.Add (self.browse_button, 0, wxEXPAND)
		esizer.Add (10, 1)
		box = wxStaticBox (self, -1, '')
		bsizer1 = wxStaticBoxSizer (box, wxVERTICAL)
		bsizer1.Add (1, 10)
		bsizer1.Add (lsizer, 0, wxEXPAND)
		bsizer1.Add (1, 5)
		bsizer1.Add (esizer, 0, wxEXPAND)
		bsizer1.Add (1, 10)
		hsizer1 = wxBoxSizer (wxHORIZONTAL)
		hsizer1.Add (10, 1)
		hsizer1.Add (bsizer1, 1, wxEXPAND)
		hsizer1.Add (10, 1)

		label = wxStaticText (self, -1, _("Number of Logins allowed:"))
		self.unlimited_button = wxRadioButton (self, self.idRADIO_UNLIMITED,
		                                             _("Unlimited"))
		self.once_button = wxRadioButton (self, self.idRADIO_ONCE,
		                                        _("Login Only Once"))
		self.limit_button = wxRadioButton (self, self.idRADIO_LIMIT,
		                                         _("Limited"))
		self.limit_entry = wxTextCtrl (self, -1, size=wxSize (35, -1))
		if opts and opts['limit']:
			limit = opts['limit']
			if limit == 0:
				self.unlimited_button.SetValue (true)
			elif limit == 1:
				self.once_button.SetValue (true)
			else:
				self.limit_button.SetValue (true)
				self.limit_entry.SetValue (str (opts['limit']))

		lsizer = wxBoxSizer (wxHORIZONTAL)
		lsizer.Add (10, 1, 1, wxEXPAND)
		lsizer.Add (label, 0, wxEXPAND)
		lsizer.Add (10, 1, 1, wxEXPAND)
		rsizer = wxBoxSizer (wxHORIZONTAL)
		rsizer.Add (self.unlimited_button, 0, wxEXPAND)
		rsizer.Add (10, 1, 1, wxEXPAND)
		rsizer.Add (self.once_button, 0, wxEXPAND)
		rsizer.Add (10, 1)
		rsizer.Add (self.limit_button, 0, wxEXPAND)
		rsizer.Add (10, 1)
		rsizer.Add (self.limit_entry, 0, wxEXPAND)
		rsizer.Add (10, 1, 1, wxEXPAND)
		box = wxStaticBox (self, -1, '')
		bsizer2 = wxStaticBoxSizer (box, wxVERTICAL)
		bsizer2.Add (1, 10)
		bsizer2.Add (lsizer, 0, wxEXPAND)
		bsizer2.Add (1, 5)
		bsizer2.Add (rsizer, 0, wxEXPAND)
		bsizer2.Add (1, 10)
		hsizer2 = wxBoxSizer (wxHORIZONTAL)
		hsizer2.Add (10, 1)
		hsizer2.Add (bsizer2, 1, wxEXPAND)
		hsizer2.Add (10, 1)

		box = wxStaticBox (self, -1, '')
		boxsizer = wxStaticBoxSizer (box, wxVERTICAL)
		boxsizer.Add (1, 10)
		boxsizer.Add (hsizer1, 0, wxEXPAND)
		boxsizer.Add (1, 10, 1, wxEXPAND)
		boxsizer.Add (hsizer2, 0, wxEXPAND)
		boxsizer.Add (1, 10)

		sizer = wxFlexGridSizer (3, 3)
		sizer.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 10), (boxsizer, 0, wxEXPAND), (10, 10),
			(10, 10), (10, 10), (10, 10),
		])
		sizer.AddGrowableCol (1)
		sizer.AddGrowableRow (1)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)
		sizer.Fit (self)
		sizer.SetSizeHints (self)

	def onBrowseClicked (self, event):
		path = messages.displayFileDialog (self, _("Ftpcube - Browse"))
		if path:
			self.local_entry.SetValue (path)

	def getOpts (self):
		limit = None
		if self.once_button.GetValue ():
			limit = 1
		elif self.limit_button.GetValue ():
			text = self.limit_entry.GetValue ()
			if text:
				limit = int (text)

		return {
			'limit' : limit,
			'localdir' : self.local_entry.GetValue (),
		}

class SettingsTab (wxPanel):

	def __init__ (self, parent, opts=None):
		wxPanel.__init__ (self, parent, -1)

		label1 = wxStaticText (self, -1, _("Connection Retries"))
		label2 = wxStaticText (self, -1, _("Idle Timeout (sec)"))
		label3 = wxStaticText (self, -1, _("Connect Timeout (sec)"))
		label4 = wxStaticText (self, -1, _("Login Retry Delay (sec)"))
		self.retry_entry = wxTextCtrl (self, -1)
		if opts and opts['retries']:
			self.retry_entry.SetValue (str (opts['retries']))
		self.idle_entry = wxTextCtrl (self, -1)
		if opts and opts['main_idle']:
			self.idle_entry.SetValue (str (opts['main_idle']))
		self.timeout_entry = wxTextCtrl (self, -1)
		if opts and opts['timeout']:
			self.timeout_entry.SetValue (str (opts['timeout']))
		self.delay_entry = wxTextCtrl (self, -1)
		if opts and opts['delay']:
			self.delay_entry.SetValue (str (opts['delay']))
		fsizer = wxFlexGridSizer (9, 2)
		fsizer.AddMany ([
			(1, 10), (1, 10),
			(label1, 0, wxEXPAND), (label2, 0, wxEXPAND),
			(1, 5), (1, 5),
			(self.retry_entry, 0, wxEXPAND), (self.idle_entry, 0, wxEXPAND),
			(1, 5), (1, 5),
			(label3, 0, wxEXPAND), (label4, 0, wxEXPAND),
			(1, 5), (1, 5),
			(self.timeout_entry, 0, wxEXPAND), (self.delay_entry, 0, wxEXPAND),
			(1, 10), (1, 10),
		])
		fsizer.SetHGap (40)
		hsizer = wxBoxSizer (wxHORIZONTAL)
		hsizer.Add (1, 1, 1, wxEXPAND)
		hsizer.Add (fsizer, 0, wxEXPAND)
		hsizer.Add (1, 1, 1, wxEXPAND)
		box = wxStaticBox (self, -1, '')
		bsizer1 = wxStaticBoxSizer (box, wxVERTICAL)
		bsizer1.Add (hsizer, 1, wxEXPAND)
		hsizer1 = wxBoxSizer (wxHORIZONTAL)
		hsizer1.Add (10, 1)
		hsizer1.Add (bsizer1, 1, wxEXPAND)
		hsizer1.Add (10, 1)

		self.passive_button = wxCheckBox (self, -1, _("Use Passive Transfers"), style=wxNO_BORDER)
		if opts and opts['passive']:
			self.passive_button.SetValue (true)
		hsizer = wxBoxSizer (wxHORIZONTAL)
		hsizer.Add (10, 1, 1, wxEXPAND)
		hsizer.Add (self.passive_button, 0, wxEXPAND)
		hsizer.Add (10, 1, 1, wxEXPAND)
		box = wxStaticBox (self, -1, '')
		bsizer2 = wxStaticBoxSizer (box, wxVERTICAL)
		bsizer2.Add (hsizer, 1, wxEXPAND | wxALIGN_CENTER_VERTICAL)
		hsizer2 = wxBoxSizer (wxHORIZONTAL)
		hsizer2.Add (10, 1)
		hsizer2.Add (bsizer2, 1, wxEXPAND)
		hsizer2.Add (10, 1)

		box = wxStaticBox (self, -1, '')
		boxsizer = wxStaticBoxSizer (box, wxVERTICAL)
		boxsizer.Add (1, 10)
		boxsizer.Add (hsizer1, 0, wxEXPAND)
		boxsizer.Add (1, 5, 1, wxEXPAND)
		boxsizer.Add (hsizer2, 0, wxEXPAND)
		boxsizer.Add (1, 10)

		sizer = wxFlexGridSizer (3, 3)
		sizer.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 10), (boxsizer, 0, wxEXPAND), (10, 10),
			(10, 10), (10, 10), (10, 10),
		])
		sizer.AddGrowableCol (1)
		sizer.AddGrowableRow (1)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)
		sizer.Fit (self)
		sizer.SetSizeHints (self)

	def getOpts (self):
		return {
			'retries' : int (self.retry_entry.GetValue ()),
			'main_idle' : int (self.idle_entry.GetValue ()),
			'timeout' : int (self.timeout_entry.GetValue ()),
			'delay' : int (self.delay_entry.GetValue ()),
			'passive' : self.passive_button.GetValue (),
		}
