#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import main
import threads
import messages
import events
import protocol
import archiver
import txtwrapper
import util
import icons.thread_idle
import icons.clear
import icons.load_queue
import icons.save_queue
import icons.main_thread
import icons.check_archive
import icons.resubmit_job
import icons.thread_idle
import icons.thread_connect
import icons.thread_listing
import icons.thread_abort
import icons.thread_action
import icons.thread_download

import os.path
from wxPython.wx import *

class ControlWindow (wxPanel):

	def __init__ (self, parent):
		if __debug__:
			print "Making control window."

		wxPanel.__init__ (self, parent, -1, style=wxNO_BORDER)

		subsizer = self.makeStatusBar ()

		# Generate the notebook
		self.notebook = wxNotebook (self, -1)
		wxNotebookSizer (self.notebook)

		self.queue_tab = QueueTab (self.notebook)
		self.notebook.AddPage (self.queue_tab, _("Queue"))

		self.threads_tab = ThreadsTab (self.notebook)
		self.notebook.AddPage (self.threads_tab, _("Threads"))

		self.download_tab = DownloadTab (self.notebook)
		self.notebook.AddPage (self.download_tab, _("Downloads"))

		self.failure_tab = FailureTab (self.notebook)
		self.notebook.AddPage (self.failure_tab, _("Failures"))

		# Start with the threads tab
		self.notebook.SetSelection (1)

		# Set up the notebook and status bar layout
		sizer = wxBoxSizer (wxVERTICAL)
		sizer.Add (self.notebook, 1, wxEXPAND)
		sizer.Add (subsizer, 0, wxEXPAND)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)

		# Register ourselves to update the queue status bar
		events.EVT_QUEUE (self, self.onQueueEvent)
		main.getApp ().registerEventListener (events.wxEVT_QUEUE, self)
		events.EVT_DEQUEUE (self, self.onDequeueEvent)
		main.getApp ().registerEventListener (events.wxEVT_DEQUEUE, self)

	def makeStatusBar (self):
		self.queue_label = wxStaticText (self, -1, _("%(cnt)d Queued")
		                                           %{ 'cnt' : 0 })
		self.queue_progress = wxGauge (self, 0, 1)
		self.queue_progress.SetSize ((-1, 7))

		# Start with no progress and with the progress bar hidden
		self.progress = (0, 0)
		self.queue_progress.Show (false)

		# Manage the layout so the bars are centered
		vsizer1 = wxBoxSizer (wxVERTICAL)
		vsizer1.Add (0, 0, 1)
		vsizer1.Add (self.queue_label, 0, wxEXPAND)
		vsizer1.Add (0, 0, 1)

		vsizer2 = wxBoxSizer (wxVERTICAL)
		vsizer2.Add (0, 0, 1)
		vsizer2.Add (self.queue_progress, 1, wxEXPAND)
		vsizer2.Add (0, 0, 1)

		sizer = wxBoxSizer (wxHORIZONTAL)
		sizer.Add (10, 0)
		sizer.Add (vsizer1, 0, wxEXPAND)
		sizer.Add (10, 0)
		sizer.Add (vsizer2, 1, wxEXPAND)
		sizer.Add (10, 0)
		return sizer

	def getQueueTab (self):
		return self.queue_tab

	def getThreadTab (self):
		return self.threads_tab

	def getDownloadTab (self):
		return self.download_tab

	def getFailureTab (self):
		return self.failure_tab

	def onQueueEvent (self, event):
		# Display the progress bar if it's hidden. Note we don't set the range
		# the first time since it defaults to 1 so the wxGauge doesn't complain
		if not self.queue_progress.IsShown ():
			self.queue_progress.Show (true)
		else:
			self.queue_progress.SetRange (self.queue_progress.GetRange () + 1)

		queued = self.queue_progress.GetRange () - self.queue_progress.GetValue ()
		self.queue_label.SetLabel (_("%(cnt)d Queued") %{ 'cnt' : queued })

	def onDequeueEvent (self, event):
		self.queue_progress.SetValue (self.queue_progress.GetValue () + 1)
		queued = self.queue_progress.GetRange () - self.queue_progress.GetValue ()
		self.queue_label.SetLabel (_("%(cnt)d Queued") %{ 'cnt' : queued })

		# Check if we've emptied the queue and should hide the progress bar
		if self.queue_progress.GetValue () == self.queue_progress.GetRange ():
			self.queue_progress.SetValue (0)
			self.queue_progress.SetRange (1)
			self.queue_progress.Show (false)

class QueueTab (wxPanel):

	idTOOL_ENABLE = wxNewId ()
	idTOOL_CLEAR  = wxNewId ()
	idTOOL_LOAD   = wxNewId ()
	idTOOL_SAVE   = wxNewId ()

	idQUEUE_LIST  = wxNewId ()

	def __init__ (self, parent):
		if __debug__:
			print "Making queuing panel."

		wxPanel.__init__ (self, parent, -1)
		self.headers = [ _("Host"), _("Filename"), _("Size"), _("Attempt") ]


		self.toolbar = self.makeToolBar ()
		isizer = wxFlexGridSizer (3, 2)
		isizer.AddMany ([
			(5, 4), (5, 4),
			(5, 4), (self.toolbar, 1, wxEXPAND),
			(5, 4), (5, 4),
		])
		isizer.AddGrowableCol (1)

		# Create the Queuing list
		self.list = wxListCtrl (self, self.idQUEUE_LIST,
		                              style=wxLC_REPORT | wxSUNKEN_BORDER)
		for i, name in zip (range (len (self.headers)), self.headers):
			self.list.InsertColumn (i, name)
		self.list.SetColumnWidth (0, 80)
		self.list.SetColumnWidth (1, 150)
		self.list.SetColumnWidth (2, 80)
		self.list.SetColumnWidth (3, 60)

		# Set up the master sizer
		sizer = wxBoxSizer (wxVERTICAL)
		sizer.Add (isizer, 0, wxEXPAND)
		sizer.Add (self.list, 1, wxEXPAND)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)

		# Initialize a dict to lookup events
		self.event_list = [ ]

		# Set the disable flag to false
		self.disabled = 0

		# Implement the queuing interface - by event
		events.EVT_QUEUE (self, self.onQueueEvent)
		main.getApp ().registerEventListener (events.wxEVT_QUEUE, self)

	def makeToolBar (self):
		# Create toolbar
		toolbar = wxToolBar (self, -1)
		toolbar.SetToolBitmapSize (wxSize (20, 20))
		bitmap = icons.thread_idle.getBitmap ()
		toolbar.AddTool (self.idTOOL_ENABLE, bitmap,
		                 isToggle=true,
		                 shortHelpString=_("Toggle Queuing Process"))
		util.AddLineSeparator (toolbar, 12)
		bitmap = icons.clear.getBitmap ()
		toolbar.AddTool (self.idTOOL_CLEAR, bitmap,
		                 shortHelpString=_("Clear Queue"))
		util.AddLineSeparator (toolbar, 12)
		bitmap = icons.load_queue.getBitmap ()
		toolbar.AddTool (self.idTOOL_LOAD, bitmap,
		                 shortHelpString=_("Load Queue"))
		bitmap = icons.save_queue.getBitmap ()
		toolbar.AddTool (self.idTOOL_SAVE, bitmap,
		                 shortHelpString=_("Save Queue"))

		EVT_TOOL (toolbar, self.idTOOL_ENABLE, self.onToggle)
		EVT_TOOL (toolbar, self.idTOOL_CLEAR, self.onClear)
		EVT_TOOL (toolbar, self.idTOOL_LOAD, self.onLoad)
		EVT_TOOL (toolbar, self.idTOOL_SAVE, self.onSave)

		toolbar.Realize ()
		return toolbar

	def onQueueEvent (self, event):
		if __debug__:
			print "Added item to transfer queue: [%s:%s - %s]" \
			      %(event.host, event.port, event.file)
		self.addListItem (event.host, event.file, event.size, event.attempt)
		self.event_list.append (event)

	def addListItem (self, host, file, size, attempt):
		index = self.list.GetItemCount ()
		self.list.InsertStringItem (index, host)
		self.list.SetStringItem (index, 1, file)
		self.list.SetStringItem (index, 2, str (size))
		self.list.SetStringItem (index, 3, str (attempt))

	def getListItem (self, index):
		entry = [ ]
		for i in range (4):
			entry.append (util.getColumnText (self.list, index, i))
		return entry

	def onToggle (self, event):
		self.disabled = (self.disabled + 1) % 2

	def isProcessingDisabled (self):
		return self.disabled

	def onClear (self, event):
		self.list.Freeze ()
		self.list.DeleteAllItems ()
		self.list.Thaw ()

	def onLoad (self, event):
		path = messages.displayFileDialog (self, _("Ftpcube - Load Queue From File"))
		if not path:
			return path

		try:
			file = None
			try:
				file = open (path, 'r')

				line = file.readline ()
				while line:
					server, filename, size, attempt = line.split ('\t')
					self.addListItem (server, filename, size, attempt)
					line = file.readline ()
			except IOError, strerror:
				messages.displayErrorDialog (self,
					_("Error reading queue data from file: %(err)s")
					%{ 'err' : strerror })
				return
		finally:
			if file:
				file.close ()

	def onSave (self, event):
		path = messages.displayFileDialog (self, _("Ftpcube - Save Queue to File"))
		if not path:
			return

		try:
			file = None
			try:
				file = open (path, 'w')

				for i in range (self.list.GetItemCount ()):
					entry = self.getListItem (i)
					string = '\t'.join (entry)
					file.write ("%s\n" %string)
			except IOError, strerror:
				messages.displayErrorDialog (self,
					_("Error writing queue data to file: %(err)s")
					%{ 'err' : strerror})
				return
		finally:
			if file:
				file.close ()


	def splitEventKey (self, key):
		server, file, size, attempt = key.split (':')
		return (server, file, long (size), int (attempt))

	def hasItems (self):
		return self.list.GetItemCount ()

	def peakNextItem (self):
		return self.event_list[0]

	def getNextItem (self):
		event = self.event_list.pop (0)
		self.list.DeleteItem (0)
		# Let everybody know we've dequeued the event
		main.getApp ().postEvent (events.wxDequeueEvent (**event.toHash ()))
		return event

class ThreadsTab (wxPanel):

	idTOOL_MAINTHREAD = wxNewId ()
	idTIMER           = wxNewId ()
	idNO_THREADS      = wxNewId ()

	WAKEUP_TIME = 1000

	def __init__ (self, parent):
		if __debug__:
			print "Making thread panel."

		wxPanel.__init__ (self, parent, -1)

		self.toolbar = self.makeToolBar ()
		isizer = wxBoxSizer (wxHORIZONTAL)
		isizer.Add (5, 4)
		isizer.Add (self.toolbar, 1, wxEXPAND)

		self.thread_label = wxStaticText (self, -1, '', style=wxALIGN_CENTER)
		self.speed_label = wxStaticText (self, -1, '', style=wxALIGN_CENTER)
		self.updateTransferTotals (0, 0) # Set the defaults

		tsizer = wxFlexGridSizer (2, 3)
		tsizer.AddMany ([
			(20, 0), (self.thread_label, 1, wxEXPAND), (20, 0),
			(20, 0), (self.speed_label, 1, wxEXPAND), (20, 0),
		])
		tsizer.AddGrowableCol (1)

		box = wxStaticBox (self, -1, '')
		self.thread_win = wxScrolledWindow (self, -1, style=wxNO_BORDER | wxVSCROLL)
		self.thread_sizer = wxBoxSizer (wxVERTICAL)
		self.thread_win.SetAutoLayout (true)
		self.thread_win.SetSizer (self.thread_sizer)
		bsizer = wxStaticBoxSizer (box, wxVERTICAL)
		bsizer.Add (self.thread_win, 1, wxEXPAND | wxALL)
		wsizer = wxBoxSizer (wxHORIZONTAL)
		wsizer.Add (10, 0)
		wsizer.Add (bsizer, 1, wxEXPAND)
		wsizer.Add (10, 0)

		# Set up the master sizer
		sizer = wxBoxSizer (wxVERTICAL)
		sizer.Add (5, 5)
		sizer.Add (isizer, 0, wxEXPAND)
		sizer.Add (5, 5)
		sizer.Add (tsizer, 0, wxEXPAND)
		sizer.Add (10, 5)
		sizer.Add (wsizer, 1, wxEXPAND)
		sizer.Add (10, 10)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)

		# Create the thread list
		self.thread_list = [ ]

		# Set the main thread flag to false
		self.main_thread_only = 0

		# Set up the idle timer
		self.timer = wxTimer (self, self.idTIMER)
		EVT_TIMER (self, self.idTIMER, self.onTimer)
		self.timer.Start (self.WAKEUP_TIME)

		# Handle event notifications from the thread manager
		events.EVT_THREAD (self, self.onThreadEvent)
		thread_mgr = main.getApp ().getThreadManager ()
		thread_mgr.addListener (self)

	def makeToolBar (self):
		opts = main.getApp ().getConfig ().getOpts ()

		# Create the toolbar
		toolbar = wxToolBar (self, -1)
		toolbar.SetToolBitmapSize (wxSize (20, 20))

		self.no_threads = wxSpinCtrl (toolbar, self.idNO_THREADS, size=wxSize (50, -1))
		self.no_threads.SetRange (1, 16)
		if opts.has_key ('max_threads'):
			self.no_threads.SetValue (opts['max_threads'])
		toolbar.AddControl (self.no_threads)
		EVT_SPINCTRL (toolbar, self.idNO_THREADS, self.onNoThreadUpdate)

		label = wxStaticText (toolbar, -1, _("max threads"))
		toolbar.AddControl (label)
		util.AddLineSeparator (toolbar, 12)
		bitmap = icons.main_thread.getBitmap ()
		toolbar.AddTool (self.idTOOL_MAINTHREAD, bitmap,
		                 isToggle=true,
		                 shortHelpString=_("Only use main thread for transfers"))

		EVT_TOOL (toolbar, self.idTOOL_MAINTHREAD, self.onMainThreadToggle)

		toolbar.Realize ()
		return toolbar

	def onNoThreadUpdate (self, event):
		value = self.no_threads.GetValue ()
		config = main.getApp ().getConfig ().getOpts ()
		config['max_threads'] = value

	def onMainThreadToggle (self, event):
		self.main_thread_only = (self.main_thread_only + 1) % 2

	def useMainThreadOnly (self):
		return self.main_thread_only

	def getQueue (self):
		ctrl_win = main.getApp ().getMainWindow ().getControlWindow ()
		return ctrl_win.getQueueTab ()

	def onTimer (self, event):
		# Check queue contents for a new event to process
		queue = self.getQueue ()
		while not queue.isProcessingDisabled () and queue.hasItems ():
			thread = self.findAvailableThread ()
			if thread:
				event = queue.getNextItem ()
				self.processEvent (thread, event)
			else:
				break

		total_rate = 0.0
		for t in self.thread_list:
			if not t.busy ():
				t.updateIdle ()
				if t.finished ():
					self.removeThreadEntry (t.getId ())
			elif t.isTransfering ():
				t.updateTransferEstimate ()
				total_rate = total_rate + t.getEntryThread ().getTransferSpeed ()
		no_threads = len (self.thread_list)
		self.updateTransferTotals (no_threads, total_rate)

	def processEvent (self, thread, event):
		real_thread = thread.getEntryThread ()
		if event.flags[0] in '-f':
			real_thread.initiateTransfer (event)
		elif event.flags[0]  == 'd':
			real_thread.recurseDirectory (event)
		else:
			real_thread.recurseLink (event)

	def findAvailableThread (self):
		thread = None
		thread_mgr = main.getApp ().getThreadManager ()
		event = self.getQueue ().peakNextItem ()

		try:
			if self.useMainThreadOnly ():
				entry = [ x for x in self.thread_list if x.getId () == 0 ]
				entry = entry.pop (0)
				if not entry.busy ():
					if entry.getHost () == event.host and \
					   entry.getPort () == event.port:
						thread = entry
					else:
						if len (self.thread_list) < self.no_threads.GetValue () \
						   and self.withinLogonLimit (event.host):
							thread = self.createThread (event)
			else:
				entries = [ x for x in self.thread_list if x.getId () != 0 ]
				for t in entries:
					if not t.busy ():
						if t.getHost () == event.host and \
						   t.getPort () == event.port:
							thread = t
						else:
							t.onCancelThread (None)
							break
				if thread is None and \
				   len (self.thread_list) < self.no_threads.GetValue () and \
				   self.withinLogonLimit (event.host):
						thread = self.createThread (event)
		finally:
			return thread

	def withinLogonLimit (self, host):
		host_list = [ x.getEntryThread ().getHost ().lower () for x in self.thread_list ]
		cnt = host_list.count (host)
		if cnt:
			thread = self.thread_list[host_list.index (host)].getEntryThread ()
			thread_opts = thread.getOpts ()
			max_logons = thread_opts['limit']
			if max_logons and max_logons <= cnt:
				return False
		return True

	def createThread (self, event):
		thread_mgr = main.getApp ().getThreadManager ()
		config = main.getApp ().getConfig ().getOpts ()

		opts = { }
		opts.update (config)
		opts.update (event.toHash ())

		# Fetch the logging objects
		clogger, dlogger, ulogger = main.getApp ().getLoggers ()

		try:
			new_id = thread_mgr.newTransferThread (
				opts=opts,
				logger=None,
				download_logger=dlogger,
				upload_logger=ulogger,
			)
			# Set the transfer state of the child thread so the default
			# failure mechanism knows which event we're talking about
			thread_mgr.getThread (new_id).saveTransferState (event)
		except Exception, strerror:
			if __debug__:
				print "Error creating new transfer thread: %s" %strerror
			return None

		# Make sure we received the new event
		self.timer.Stop ()
		threadlist = None
		if not threadlist:
			wxYield ()
			threadlist = [ x for x in self.thread_list if x.getId () == new_id ]
		self.timer.Start (self.WAKEUP_TIME)
		thread_panel =  threadlist.pop (0)

		# Start the connection
		thread_panel.getEntryThread ().initiateConnect ()
		return thread_panel

	def onThreadEvent (self, event):
		thread_ids = [ x.getId () for x in self.thread_list ]

		# Check if the thread exists
		try:
			index = thread_ids.index (event.id)
		except ValueError:
			index = None

		# Post the event to the appropriate thread GUI piece if the
		# thread exists
		if index is not None:
			wxPostEvent (self.thread_list[index], event)
			return

		# Otherwise, let's handle the event ourselves
		if event.kind == events.wxThreadEvent.EVT_CREATE:
			self.addThreadEntry (event.id)
		elif event.kind == events.wxThreadEvent.EVT_DESTROY:
			self.removeThreadEntry (event.id)

	def addThreadEntry (self, id):
		entry = ThreadEntry (self.thread_win, id)
		self.thread_list.append (entry)
		self.thread_sizer.Add (entry, 0, wxEXPAND | wxADJUST_MINSIZE)
		self.thread_sizer.Layout ()

	def removeThreadEntry (self, id):
		found = None
		for thread in self.thread_list:
			if thread.getId () == id:
				found = thread
				break

		if found:
			self.thread_list.remove (found)
			self.thread_sizer.Remove (found)
			found.Destroy ()
			self.thread_sizer.Layout ()

	def updateTransferTotals (self, threads, speed):
		self.thread_label.SetLabel (_("%(no)d Threads Running") %{ 'no' : threads })
		self.speed_label.SetLabel (_("Total Speed: %(kb)0.02f Kb/s") %{ 'kb' : speed })
		self.Layout ()

class ThreadEntry (wxPanel):

	idCANCEL_TRANSFER = wxNewId ()
	idCANCEL_THREAD   = wxNewId ()
	idTOGGLE_TIMEOUT  = wxNewId ()

	def __init__ (self, parent, thread_id):
		if __debug__:
			print "Making thread entry status box."

		wxPanel.__init__ (self, parent, -1)

		# Set instance variables
		self.thread_id = thread_id
		self.thread = self.getEntryThread ()

		self.loadBitmaps ()

		self.progress_bar = wxGauge (self, -1, 1, size=wxSize (200, 10))
		self.image = wxStaticBitmap (self, -1, self.idle_bitmap, size=wxSize (20, 20))
		name = ''
		if self.getThreadName ():
			label_text = "%s\n" %self.getThreadName () + _("Initializing Thread...")
		else:
			label_text = _("Initializing Thread...") + "\n"
		self.label = txtwrapper.wxStaticWrapText (self, -1, '')
		self.label.SetLabel (label_text)
		self.line = wxStaticLine (self, -1)

		sizer = wxBoxSizer (wxHORIZONTAL)
		sizer.Add (1, 5, 1, wxEXPAND)
		sizer.Add (self.progress_bar, 0, wxEXPAND)
		sizer.Add (1, 5, 1, wxEXPAND)
		self.psizer = wxBoxSizer (wxVERTICAL)
		self.psizer.Add (1, 5, 0, wxEXPAND)
		self.psizer.Add (sizer, 0, wxEXPAND)
		self.psizer.Add (1, 5, 0, wxEXPAND)

		self.lsizer = wxBoxSizer (wxHORIZONTAL)
		self.lsizer.Add (5, 1, 0, wxEXPAND)
		self.lsizer.Add (self.image, 0, wxALIGN_CENTER_VERTICAL)
		self.lsizer.Add (10, 1, 0, wxEXPAND)
		self.lsizer.Add (self.label, 1, wxADJUST_MINSIZE)

		self.slsizer = wxBoxSizer (wxHORIZONTAL)
		self.slsizer.Add (self.line, 1, wxADJUST_MINSIZE)

		self.sizer = wxBoxSizer (wxVERTICAL)
		self.sizer.Add (self.lsizer, 1, wxEXPAND)
		self.sizer.Add (self.slsizer, 0, wxEXPAND)
		self.SetAutoLayout (true)
		self.SetSizer (self.sizer)
		self.sizer.SetSizeHints (self)
		self.sizer.Fit (self)

		# Hide the progress bar intially. We don't even add it into
		# the sizer, since it takes up space if we do
		self.progress_bar.Show (false)

		# Handle thread notifications from the thread manager
		events.EVT_THREAD (self, self.onThreadEvent)

		# Enable pop-up menu
		EVT_RIGHT_DOWN (self, self.onRightClick)
		EVT_RIGHT_DOWN (self.image, self.onRightClick)
		EVT_RIGHT_DOWN (self.label, self.onRightClick)

	def loadBitmaps (self):
		self.idle_bitmap = icons.thread_idle.getBitmap ()
		self.connect_bitmap = icons.thread_connect.getBitmap ()
		self.listing_bitmap = icons.thread_listing.getBitmap ()
		self.abort_bitmap = icons.thread_abort.getBitmap ()
		self.action_bitmap = icons.thread_action.getBitmap ()
		self.transfer_bitmap = icons.thread_download.getBitmap ()

	def onRightClick (self, event):
		menu = self.makePopupMenu ()
		self.PopupMenu (menu, wxPoint (event.m_x, event.m_y))

	def makePopupMenu (self):
		menu = wxMenu ()
		menu.Append (self.idCANCEL_TRANSFER, _("Cancel Transfer"),
		             _("Cancel this thread's transfer"))
		menu.Append (self.idCANCEL_THREAD, _("Cancel Thread"),
		             _("Stops the current thread"))
		menu.AppendSeparator ()

		if self.getEntryThread ().isIdleEnabled ():
			menu.Append (self.idTOGGLE_TIMEOUT, _("Disable Idle Timeout"),
			             _("Disables timeout while idling"))
		else:
			menu.Append (self.idTOGGLE_TIMEOUT, _("Enable Idle Timeout"),
			             _("Enables timeout while idling"))

		EVT_MENU (self, self.idCANCEL_TRANSFER, self.onCancelTransfer)
		EVT_MENU (self, self.idCANCEL_THREAD, self.onCancelThread)
		EVT_MENU (self, self.idTOGGLE_TIMEOUT, self.onToggleTimeout)

		return menu

	def updateVisualDisplay (self):
		self.sizer.SetSizeHints (self)
		self.Layout ()

	def getId (self):
		return self.thread_id

	def getEntryThread (self):
		thread_mgr = main.getApp ().getThreadManager ()
		return thread_mgr.getThread (self.thread_id)

	def getHost (self):
		return self.getEntryThread ().getHost ()

	def getPort (self):
		return self.getEntryThread ().getPort ()

	def Destroy (self):
		self.thread.cancelConnect ()
		main.getApp ().getThreadManager ().removeThread (self.getId ())
		wxPanel.Destroy (self)

	def onThreadEvent (self, event):
		if event.kind == events.wxThreadEvent.EVT_CONNECT:
			if __debug__:
				print "Thread [%d] received event: [CONNECT]" %self.getId ()
			self.updateConnect ()
		elif event.kind == events.wxThreadEvent.EVT_LIST:
			if __debug__:
				print "Thread [%d] received event: [LIST]" %self.getId ()
			self.updateList ()
		elif event.kind == events.wxThreadEvent.EVT_CWD:
			if __debug__:
				print "Thread [%d] received event: [CWD]" %self.getId ()
			self.updateCwd ()
		elif event.kind == events.wxThreadEvent.EVT_RENAME:
			if __debug__:
				print "Thread [%d] received event: [RENAME]" %self.getId ()
			self.updateRename (event.data[0], event.data[1])
		elif event.kind == events.wxThreadEvent.EVT_DELETE:
			if __debug__:
				print "Thread [%d] received event: [DELETE]" %self.getId ()
			self.updateDelete ()
		elif event.kind == events.wxThreadEvent.EVT_MKDIR:
			if __debug__:
				print "Thread [%d] received event: [MKDIR]" %self.getId ()
			self.updateMkdir ()
		elif event.kind == events.wxThreadEvent.EVT_RMDIR:
			if __debug__:
				print "Thread [%d] received event: [RMDIR]" %self.getId ()
			self.updateRmdir ()
		elif event.kind == events.wxThreadEvent.EVT_CHMOD:
			if __debug__:
				print "Thread [%d] received event: [CHMOD]" %self.getId ()
			self.updateChmod ()
		else:
			if __debug__:
				print "Thread [%d] receiver unknown event of type: [%d]" \
				       %(self.getId (), event.kind)

	def onCancelTransfer (self, event):
		opts = self.thread.getOpts ()
		self.label.SetLabel (self.getThreadName () +
		                     _("Host: %(host)s\nCancelling Transfer...\n")
		                     %{ 'host' : opts['host'] })
		self.image.SetBitmap (self.abort_bitmap)
		self.updateVisualDisplay ()
		self.thread.abort ()

	def onCancelThread (self, event):
		if self.busy ():
			self.onCancelTransfer (event)
		self.thread.setDone ()

	def onToggleTimeout (self, event):
		bool = self.thread.isIdleEnabled ()
		# Perform the toggle
		self.thread.setIdleEnable (not bool or bool and not bool)

	def getThreadName (self):
		name = self.thread.getName ()
		if name:
			return "[%s] " %name
		return ''

	def busy (self):
		return self.thread.busy ()

	def finished (self):
		return self.thread.finished ()

	def isTransfering (self):
		return self.thread.isTransfering ()

	def updateConnect (self):
		opts = self.thread.getOpts ()
		self.label.SetLabel (self.getThreadName () + _("Host: %(host)s\nConnecting...")
		                     %{ 'host' : opts['host'] })
		self.image.SetBitmap (self.connect_bitmap)
		self.updateVisualDisplay ()

	def updateList (self):
		opts = self.thread.getOpts ()
		self.label.SetLabel (self.getThreadName () +
		                     _("Host: %(host)s\nRetrieving Directory Listing.")
		                     %{ 'host' : opts['host'] })
		self.image.SetBitmap (self.listing_bitmap)
		self.updateVisualDisplay ()

	def updateCwd (self):
		opts = self.thread.getOpts ()
		self.label.SetLabel (self.getThreadName () +
		                     _("Host: %(host)s\nChanging Directory...")
		                     %{ 'host' : opts['host'] })
		self.image.SetBitmap (self.action_bitmap)
		self.updateVisualDisplay ()

	def updateRename (self, old, new):
		opts = self.thread.getOpts ()
		self.label.SetLabel (self.getThreadName () +
		                     _("Host: %(host)s\nRenaming %(old)s to %(new)s...\n")
		                     %{ 'host' : opts['host'], 'old' : old, 'new' : new })
		self.image.SetBitmap (self.action_bitmap)
		self.updateVisualDisplay ()

	def updateDelete (self):
		opts = self.thread.getOpts ()
		self.label.SetLabel (self.getThreadName () +
		                     _("Host: %(host)s\nDeleting File...\n")
		                     %{ 'host' : opts['host'] })
		self.image.SetBitmap (self.action_bitmap)
		self.updateVisualDisplay ()

	def updateMkdir (self):
		opts = self.thread.getOpts ()
		self.label.SetLabel (self.getThreadName () +
		                     _("Host: %(host)s\nCreating Directory...\n")
		                     %{ 'host' : opts['host'] })
		self.image.SetBitmap (self.action_bitmap)
		self.updateVisualDisplay ()

	def updateRmdir (self):
		opts = self.thread.getOpts ()
		self.label.SetLabel (self.getThreadName () +
		                     _("Host: %(host)s\nRemoving Directory...\n")
		                     %{ 'host' : opts['host'] })
		self.image.SetBitmap (self.action_bitmap)
		self.updateVisualDisplay ()

	def updateChmod (self):
		opts = self.thread.getOpts ()
		self.label.SetLabel (self.getThreadName () +
		                     _("Host: %(host)s\nApplying Permissions...\n")
		                     %{ 'host' : opts['host'] })
		self.image.SetBitmap (self.action_bitmap)
		self.updateVisualDisplay ()

	def updateIdle (self):
		opts = self.thread.getOpts ()
		if self.thread.isIdleEnabled ():
			self.thread.updateIdleTime ()
		self.hideProgressBar ()
		self.label.SetLabel (self.getThreadName () +
		                     _("Host: %(host)s\nIdle for %(time)s secs")
		                     %{ 'host' : opts['host'],
		                        'time' : self.thread.getIdleTime () })
		self.image.SetBitmap (self.idle_bitmap)
		self.updateVisualDisplay ()

	def displayProgressBar (self):
		if not self.progress_bar.IsShown ():
			self.progress_bar.Show (true)
			self.sizer.Prepend (self.psizer, 0, wxEXPAND)
			self.sizer.Layout ()
			self.updateVisualDisplay ()
			wxYield ()

	def hideProgressBar (self):
		if self.progress_bar.IsShown ():
			self.progress_bar.Show (false)

			# Workaround for removing the sizer, since the normal Remove ()
			# tries to delete the sizer upon removal
			children = [ x.GetSizer () for x in self.sizer.GetChildren () ]
			pos = children.index (self.psizer)
			item = self.sizer.GetChildren ()[pos]
			if item.IsSizer ():
				item.SetSizer (None)
			self.sizer.Remove (pos)

			self.sizer.Layout ()
			self.updateVisualDisplay ()
			wxYield ()

	def updateTransferEstimate (self):
		rate = self.thread.getTransferSpeed ()
		cur, max = self.thread.getProgress ()
		if not self.progress_bar.IsShown ():
			self.progress_bar.SetRange (max)
			self.displayProgressBar ()
		self.progress_bar.SetValue (cur)
		if rate:
			remaining = long (float (max - cur) / (rate * 1024.0))
		else:
			remaining = 0
		hrs = remaining / 3600
		min = (remaining % 3600) / 60
		sec = remaining % 60

		opts = self.thread.getOpts ()
		event = self.thread.loadTransferState ()
		self.image.SetBitmap (self.transfer_bitmap)
		if event.direction == protocol.DOWNLOAD:
			self.label.SetLabel (self.getThreadName () +
				_("Host: %(host)s\nDownloading %(file)s with %(rate).02f kb/s, %(cur)d of %(end)d bytes, %(hrs)d:%(min)2d:%(sec)2d sec left")
				%{ 'host' : event.host,
				   'file' : event.file,
				   'rate' : rate,
				   'cur' : cur,
				   'end' : max,
				   'hrs' : hrs,
				   'min' : min,
				   'sec' : sec })
		elif event.direction == protocol.UPLOAD:
			self.label.SetLabel (self.getThreadName () +
				_("Host: %(host)s\nUploading %(file)s with %(rate).02f kb/s, %(cur)d of %(end)d bytes, %(hrs)d:%(min)2d:%(sec)2d sec left")
				%{ 'host' : event.host,
				   'file' : event.file,
				   'rate' : rate,
				   'cur' : cur,
				   'end' : max,
				   'hrs' : hrs,
				   'min' : min,
				   'sec' : sec})
		self.updateVisualDisplay ()

class DownloadTab (wxPanel):

	idTOOL_CLEAR   = wxNewId ()
	idTOOL_EXPLORE = wxNewId ()

	idATTEMPTS = wxNewId ()

	def __init__ (self, parent):
		wxPanel.__init__ (self, parent, -1)
		self.status_headers = [ _("Filename"), _("Status"), _("Type"), _("Size") ]
		self.archive_headers = [ _("Filename"), _("Size") ]

		self.toolbar = self.makeToolBar ()
		isizer = wxFlexGridSizer (3, 2)
		isizer.AddMany ([
			(5, 4), (5, 4),
			(5, 4), (self.toolbar, 1, wxEXPAND),
			(5, 4), (5, 4),
		])
		isizer.AddGrowableCol (1)

		# Create the status list
		self.event_list = [ ]
		self.status_list = wxListCtrl (self, -1,
		                               style=wxLC_REPORT | wxSUNKEN_BORDER | wxLC_SINGLE_SEL)
		for i, name in zip (range (len (self.status_headers)), self.status_headers):
			self.status_list.InsertColumn (i, name)
		# Explicitly make the filename bigger
		self.status_list.SetColumnWidth (0, 125)

		# Create the archive window
		self.arch_list = [ ]
		self.archive_list = wxListCtrl (self, -1, style=wxLC_REPORT | wxSUNKEN_BORDER)
		for i, name in zip (range (len (self.archive_headers)), self.archive_headers):
			self.archive_list.InsertColumn (i, name)
		self.archive_list.SetColumnWidth (0, 200)

		# Set up the master sizer
		sizer = wxBoxSizer (wxVERTICAL)
		sizer.Add (isizer, 0, wxEXPAND)
		sizer.Add (self.status_list, 1, wxEXPAND)
		sizer.Add (self.archive_list, 0, wxEXPAND)
		sizer.SetItemMinSize (self.archive_list, -1, 200)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)

		# Implement the download interface
		events.EVT_DOWNLOAD (self, self.onDownloadEvent)
		main.getApp ().registerEventListener (events.wxEVT_DOWNLOAD, self)

		# Create an archiver instance
		self.archiver = archiver.Archiver (self)

	def makeToolBar (self):
		config = main.getApp ().getConfig ().getOpts ()

		toolbar = wxToolBar (self, -1)
		toolbar.SetToolBitmapSize (wxSize (20, 20))
		bitmap = icons.clear.getBitmap ()
		toolbar.AddTool (self.idTOOL_CLEAR, bitmap,
		                shortHelpString=_("Clear List"))
		bitmap = icons.check_archive.getBitmap ()
		toolbar.AddTool (self.idTOOL_EXPLORE, bitmap,
		                 shortHelpString=_("Explore Archive"))
		toolbar.AddSeparator ()

		self.max_attempts = wxSpinCtrl (toolbar, self.idATTEMPTS,
		                                min=1, max=16, size=wxSize (50, -1))
		if config.has_key ('retries'):
			self.max_attempts.SetValue (config['retries'])
		toolbar.AddControl (self.max_attempts)
		EVT_SPINCTRL (toolbar, self.idATTEMPTS, self.onSpinUpdate)

		label = wxStaticText (toolbar, -1, _("max attempts"))
		toolbar.AddControl (label)

		EVT_TOOL (toolbar, self.idTOOL_CLEAR, self.onClear)
		EVT_TOOL (toolbar, self.idTOOL_EXPLORE, self.onExplore)

		toolbar.Realize ()
		return toolbar

	def onSpinUpdate (self, event):
		value = self.max_attempts.GetValue ()
		config = main.getApp ().getConfig ().getOpts ()
		config['retries'] = value

	def onDownloadEvent (self, event):
		path = os.path.join (event.local_path, event.file)
		try:
			size = os.path.getsize (path)
			size = util.beautifySize (size)
		except OSError, strerror:
			size = _("Err")

		status = self.archiver.getStatus (path)
		type = self.archiver.getType (path)

		self.addStatusListItem (event.file, status, type, size)
		self.event_list.append (event)

	def addStatusListItem (self, file, status, type, size):
		index = self.status_list.GetItemCount ()
		self.status_list.InsertStringItem (index, file)
		self.status_list.SetStringItem (index, 1, status)
		self.status_list.SetStringItem (index, 2, type)
		self.status_list.SetStringItem (index, 3, size)

	def addArchiveListItem (self, file, size):
		index = self.archive_list.GetItemCount ()
		self.archive_list.InsertStringItem (index, file)
		self.archive_list.SetStringItem (index, 1, str (size))

	def onClear (self, event):
		self.status_list.Freeze ()
		self.status_list.DeleteAllItems ()
		self.status_list.Thaw ()
		self.archive_list.Freeze ()
		self.archive_list.DeleteAllItems ()
		self.archive_list.Thaw ()
		self.event_list = [ ]
		self.arch_list = [ ]

	def onExplore (self, event):
		selected = util.getSelected (self.status_list)
		for x in selected:
			item = self.event_list[x]
			unpack_loc = messages.displayDirDialog (self,
				_("Ftpcube - Select Directory to Unpack Archive: %(arch)s")
				%{ 'arch' : item.file })

			files = [ ]
			if unpack_loc:
				try:
					files = self.archiver.unpack (
						os.path.join (item.local_path, item.file),
						unpack_loc
					)
				except Exception, strerror:
					messages.displayErrorDialog (self, strerror)
					continue

				loc_len = len (unpack_loc) + 1
				for f in files:
					try:
						size = os.path.getsize (f)
					except OSError, strerror:
						size = _("Unknown")
						messages.displayErrorDialog (self, strerror)
					self.addArchiveListItem (f[loc_len:],
					                         size)
					self.arch_list.append (f)

class FailureTab (wxPanel):

	idTOOL_CLEAR    = wxNewId ()
	idTOOL_RESUBMIT = wxNewId ()

	def __init__ (self, parent):
		wxPanel.__init__ (self, parent, -1)
		self.headers = [ _("Server"), _("Filename"), _("Size"), _("Error Msg") ]

		self.toolbar = self.makeToolBar ()
		isizer = wxFlexGridSizer (3, 2)
		isizer.AddMany ([
			(5, 4), (5, 4),
			(5, 4), (self.toolbar, 1, wxEXPAND),
			(5, 4), (5, 4),
		])
		isizer.AddGrowableCol (1)

		# Create the archive window
		self.failure_list = [ ]
		self.list = wxListCtrl (self, -1, style=wxLC_REPORT | wxSUNKEN_BORDER)
		for i, name in zip (range (len (self.headers)), self.headers):
			self.list.InsertColumn (i, name)
		self.list.SetColumnWidth (0, 80)
		self.list.SetColumnWidth (1, 100)
		self.list.SetColumnWidth (2, 80)
		self.list.SetColumnWidth (3, 500)

		# Set up the master sizer
		sizer = wxBoxSizer (wxVERTICAL)
		sizer.Add (isizer, 0, wxEXPAND)
		sizer.Add (self.list, 1, wxEXPAND)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)

		events.EVT_FAILURE (self, self.onFailureEvent)
		main.getApp ().registerEventListener (events.wxEVT_FAILURE, self)

	def makeToolBar (self):
		toolbar = wxToolBar (self, -1)
		toolbar.SetToolBitmapSize (wxSize (20, 20))
		bitmap = icons.clear.getBitmap ()
		toolbar.AddTool (self.idTOOL_CLEAR, bitmap,
		                 shortHelpString=_("Clear List"))
		bitmap = icons.resubmit_job.getBitmap ()
		toolbar.AddTool (self.idTOOL_RESUBMIT, bitmap,
		                 shortHelpString=_("Resubmit Job"))

		EVT_TOOL (toolbar, self.idTOOL_CLEAR, self.onClear)
		EVT_TOOL (toolbar, self.idTOOL_RESUBMIT, self.onResubmit)

		toolbar.Realize ()
		return toolbar

	def onFailureEvent (self, event):
		event.attempt = event.attempt + 1
		if event.attempt > event.maxattempts:
			# Add the event because we've exceed max attempts
			self.addListItem (event.host, event.file, event.size, event.error)
			self.failure_list.append (event)
		else:
			# Retry the event some more
			queue_evt = events.wxQueueEvent (**event.toHash ())
			main.getApp ().postEvent (queue_evt)

	def addListItem (self, server, file, size, err_msg):
		index = self.list.GetItemCount ()
		self.list.InsertStringItem (index, server)
		self.list.SetStringItem (index, 1, file)
		self.list.SetStringItem (index, 2, str (size))
		self.list.SetStringItem (index, 3, str (err_msg))

	def onClear (self, event):
		self.list.Freeze ()
		self.list.DeleteAllItems ()
		self.list.Thaw ()
		self.failure_list = [ ]

	def onResubmit (self, event):
		selected = util.getSelected (self.list)
		for x in selected:
			evt = self.failure_list[x]

			# Reset the attempts counter
			evt.attempt = 0

			new_event = events.wxQueueEvent (**evt.toHash ())
			main.getApp ().postEvent (new_event)

			# Now remove items from the list
			self.list.Freeze ()
			for x in selected:
				self.failure_list.remove (self.failure_list[x])
				self.list.DeleteItem (x)
			self.list.Thaw ()
