#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

from wxPython.wx import *

#########################################################################
# Events for main window widgets
#########################################################################

wxEVT_DIRCHANGE = wxNewEventType ()

def EVT_DIRCHANGE (win, func):
	win.Connect (-1, -1, wxEVT_DIRCHANGE, func)

class wxDirChangeEvent (wxPyEvent):

	def __init__ (self, directory):
		wxPyEvent.__init__ (self)
		self.SetEventType (wxEVT_DIRCHANGE)
		self.directory = directory

#########################################################################
# Events for file window widgets
#########################################################################

class BrowseWindowEvent (wxPyEvent):

	EVT_LIST_UPDATE  = wxNewId ()
	EVT_LIST_REFRESH = wxNewId ()

	def __init__ (self, kind, msg):
		wxPyEvent.__init__ (self)
		self.kind = kind
		self.msg = msg

wxEVT_REMOTE_WINDOW = wxNewEventType ()

def EVT_REMOTE_WINDOW (win, func):
	win.Connect (-1, -1, wxEVT_REMOTE_WINDOW, func)

class wxRemoteWindowEvent (BrowseWindowEvent):

	EVT_CWD          = wxNewId ()

	def __init__ (self, kind, msg):
		BrowseWindowEvent.__init__ (self, kind, msg)
		self.SetEventType (wxEVT_REMOTE_WINDOW)

wxEVT_LOCAL_WINDOW = wxNewEventType ()

def EVT_LOCAL_WINDOW (win, func):
	win.Connect (-1, -1, wxEVT_LOCAL_WINDOW, func)

class wxLocalWindowEvent (BrowseWindowEvent):

	def __init__ (self, kind, msg):
		BrowseWindowEvent.__init__ (self, kind, msg)
		self.SetEventType (wxEVT_LOCAL_WINDOW)

#########################################################################
# Events for remote file window widget
#########################################################################

wxEVT_CONSOLE = wxNewEventType ()

def EVT_CONSOLE (win, func):
	win.Connect (-1, -1, wxEVT_CONSOLE, func)

class wxConsoleEvent (wxPyEvent):

	def __init__ (self, kind, msg):
		wxPyEvent.__init__ (self)
		self.SetEventType (wxEVT_CONSOLE)
		self.kind = kind
		self.msg = msg

#########################################################################
# Events for notification of thread activity to thread widgets
#########################################################################

wxEVT_THREAD = wxNewEventType ()

def EVT_THREAD (win, func):
	win.Connect (-1, -1, wxEVT_THREAD, func)

class wxThreadEvent (wxPyEvent):

	EVT_CREATE    = wxNewId ()
	EVT_DESTROY   = wxNewId ()
	EVT_CONNECT   = wxNewId ()
	EVT_LIST      = wxNewId ()
	EVT_CWD       = wxNewId ()
	EVT_RENAME    = wxNewId ()
	EVT_DELETE    = wxNewId ()
	EVT_MKDIR     = wxNewId ()
	EVT_RMDIR     = wxNewId ()
	EVT_CHMOD     = wxNewId ()

	def __init__ (self, id, kind, data=None):
		wxPyEvent.__init__ (self)
		self.SetEventType (wxEVT_THREAD)
		self.id = id
		self.kind = kind
		self.data = data

#########################################################################
# Events for interfacing with the control queue, thread, download, and
# failure widgets
#########################################################################

wxEVT_QUEUE    = wxNewEventType ()
wxEVT_DEQUEUE  = wxNewEventType ()
wxEVT_DOWNLOAD = wxNewEventType ()
wxEVT_FAILURE  = wxNewEventType ()

def EVT_QUEUE (win, func):
	win.Connect (-1, -1, wxEVT_QUEUE, func)

def EVT_DEQUEUE (win, func):
	win.Connect (-1, -1, wxEVT_DEQUEUE, func)

def EVT_DOWNLOAD (win, func):
	win.Connect (-1, -1, wxEVT_DOWNLOAD, func)

def EVT_FAILURE (win, func):
	win.Connect (-1, -1, wxEVT_FAILURE, func)

class TransferEvent (wxPyEvent):

	def __init__ (self, **kwargs):
		wxPyEvent.__init__ (self)

		self.file        = kwargs['file']
		self.host        = kwargs['host']
		self.port        = kwargs['port']
		self.username    = kwargs['username']
		self.password    = kwargs['password']
		self.remote_path = kwargs['remote_path']
		self.local_path  = kwargs['local_path']
		self.size        = kwargs['size']
		self.attempt     = kwargs['attempt']
		self.maxattempts = kwargs['maxattempts']
		self.direction   = kwargs['direction']
		self.flags       = kwargs['flags']
		self.method      = kwargs['method']

	def toHash (self):
		return {
			'file'        : self.file,
			'host'        : self.host,
			'port'        : self.port,
			'username'    : self.username,
			'password'    : self.password,
			'remote_path' : self.remote_path,
			'local_path'  : self.local_path,
			'size'        : self.size,
			'attempt'     : self.attempt,
			'maxattempts' : self.maxattempts,
			'direction'   : self.direction,
			'flags'       : self.flags,
			'method'      : self.method,
		}

class wxQueueEvent (TransferEvent):

	def __init__ (self, **kwargs):
		TransferEvent.__init__ (self, **kwargs)
		self.SetEventType (wxEVT_QUEUE)

class wxDequeueEvent (TransferEvent):

	def __init__ (self, **kwargs):
		TransferEvent.__init__ (self, **kwargs)
		self.SetEventType (wxEVT_DEQUEUE)

class wxDownloadEvent (TransferEvent):

	def __init__ (self, **kwargs):
		TransferEvent.__init__ (self, **kwargs)
		self.SetEventType (wxEVT_DOWNLOAD)

class wxFailureEvent (TransferEvent):

	def __init__ (self, **kwargs):
		TransferEvent.__init__ (self, **kwargs)
		self.SetEventType (wxEVT_FAILURE)
		self.error = kwargs['error']

	def toHash (self):
		hash = TransferEvent.toHash (self)
		hash.update ({ 'error' : self.error })
		return hash
