#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import main
import consolewin
import events

import time
import threading

class Logger:

	def __init__ (self):
		pass

	def log (self, kind, msg):
		raise NotImplementedError

class ConsoleLogger (Logger):

	def __init__ (self):
		Logger.__init__ (self)

	def log (self, kind, msg):
		event = events.wxConsoleEvent (kind, msg)
		main.getApp ().postEvent (event)

class FileLogger (Logger):

	# Lock across global file operations
	file_lock = threading.Lock ()

	def __init__ (self, *args):
		Logger.__init__ (self)

		self.format = {
			'local'  : _("LOCAL > %s\n"),
			'remote' : _("REMOTE> %s\n"),
			'error'  : _("! - %s\n"),
		}

		self.logs = [ ]
		try:
			for f in args:
				self.logs.append (open (f, 'a'))
		except IOError, strerror:
			# Close out any open files
			try:
				for f in self.logs:
					f.close ()
			except:
				pass
			raise IOError, strerror

		self.printMark ()

	def __del__ (self):
		self.closeAll ()

	def printMark (self):
		for log in self.logs:
			log.write (_("\n---------- MARK %s ----------\n\n") %time.asctime ())
			log.flush ()

	def log (self, *args):
		self.file_lock.acquire ()
		# Check if we're doing kind, msg, otherwise, just
		# use the first argument as the text message
		if len (args) == 2:
			kind, msg = args
			for log in self.logs:
				text = self.format[kind] %msg
				log.write (text)
				log.flush ()
		else:
			for log in self.logs:
				log.write (args[0])
				log.flush ()
		self.file_lock.release ()

	def closeAll (self):
		try:
			[ i.close () for i in self.logs ]
			self.logs = [ ]
		except:
			pass

class FileAndConsoleLogger (ConsoleLogger, FileLogger):

	def __init__ (self, *args):
		ConsoleLogger.__init__ (self)
		FileLogger.__init__ (self, *args)

	def log (self, kind, msg):
		ConsoleLogger.log (self, kind, msg)
		FileLogger.log (self, kind, msg)
