#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import main
import dialog
import util

# Try to get wxPyColourChooser primarily from the wxPython kit.
# If we can't find it, we'll try to import it directly
try:
	import wxPython.lib.colourchooser as wxPyColourChooser
except ImportError, strerror:
	import wxPyColourChooser

from wxPython.wx import *

class OptionWindow (dialog.DialogWindow):

	def __init__ (self, parent, opts=None):
		if __debug__:
			print "Making option window."

		dialog.DialogWindow.__init__ (self, parent, _("Ftpcube - Options"))
		self.SetIcon (main.getApp ().getAppIcon ())

		panel = self.getDialogPanel ()

		notebook = wxNotebook (panel, -1)
		notebook_sizer = wxNotebookSizer (notebook)

		self.connection_tab = ConnectionTab (notebook, opts)
		notebook.AddPage (self.connection_tab, _("Connections"))
		self.logging_tab = LoggingTab (notebook, opts)
		notebook.AddPage (self.logging_tab, _("Logging"))
		self.colors_tab = ColorsTab (notebook, opts)
		notebook.AddPage (self.colors_tab, _("Colors"))

		esizer = wxBoxSizer (wxVERTICAL)
		esizer.Add (notebook_sizer, 1, wxEXPAND)
		panel.SetAutoLayout (true)
		panel.SetSizer (esizer)
		esizer.Fit (panel)
		esizer.SetSizeHints (panel)

		self.SetSizeHints (450, -1)
		self.renderDialog ()

	def getOpts (self):
		try:
			opts = { }
			opts.update (self.connection_tab.getOpts ())
			opts.update (self.logging_tab.getOpts ())
			opts.update (self.colors_tab.getOpts ())
			return opts
		except:
			return { }

class ConnectionTab (wxPanel):

	def __init__ (self, parent, opts=None):
		wxPanel.__init__ (self, parent, -1)

		ctimeout_label = wxStaticText (self, -1, _("Connection Timeout:"))
		idle_label = wxStaticText (self, -1, _("Idle Timeout:"))
		delay_label = wxStaticText (self, -1, _("Delay Between Retries:"))
		attempts_label = wxStaticText (self, -1, _("Number of Attempts:"))
		self.ctimeout_entry = wxTextCtrl (self, -1)
		if opts and opts['timeout']:
			self.ctimeout_entry.SetValue (str (opts['timeout']))
		self.idle_entry = wxTextCtrl (self, -1)
		if opts and opts['main_idle']:
			self.idle_entry.SetValue (str (opts['main_idle']))
		self.delay_entry = wxTextCtrl (self, -1)
		if opts and opts['delay']:
			self.delay_entry.SetValue (str (opts['delay']))
		self.attempts_entry = wxTextCtrl (self, -1)
		if opts and opts['retries']:
			self.attempts_entry.SetValue (str (opts['retries']))
		slabel1 = wxStaticText (self, -1, _("(seconds)"))
		slabel2 = wxStaticText (self, -1, _("(seconds)"))
		slabel3 = wxStaticText (self, -1, _("(seconds)"))

		self.passive_box = wxCheckBox (self, -1, _("Default to Passive Transfers"))
		if opts and opts['passive']:
			self.passive_box.SetValue (true)

		tsizer = wxFlexGridSizer (5, 5)
		tsizer.AddMany ([
			(5, 1),
			(ctimeout_label, 0, wxALIGN_CENTER_VERTICAL),
			(self.ctimeout_entry, 0, wxEXPAND),
			(slabel1, 0, wxALIGN_CENTER_VERTICAL),
			(5, 1),

			(5, 1),
			(idle_label, 0, wxALIGN_CENTER_VERTICAL),
			(self.idle_entry, 0, wxEXPAND),
			(slabel2, 0, wxALIGN_CENTER_VERTICAL),
			(5, 1),

			(5, 1),
			(delay_label, 0, wxALIGN_CENTER_VERTICAL),
			(self.delay_entry, 0, wxEXPAND),
			(slabel3, 0, wxALIGN_CENTER_VERTICAL),
			(5, 1),

			(5, 1),
			(attempts_label, 0, wxALIGN_CENTER_VERTICAL),
			(self.attempts_entry, 0, wxEXPAND),
			(1, 1),
			(5, 1),
		])
		tsizer.AddGrowableCol (2)
		tsizer.SetHGap (10)
		tsizer.SetVGap (7)
		box = wxStaticBox (self, -1, _("Connection Parameters"))
		bsizer1 = wxStaticBoxSizer (box, wxVERTICAL)
		bsizer1.Add (5, 7)
		bsizer1.Add (tsizer, 1, wxEXPAND)
		bsizer1.Add (5, 10)
		bsizer1.Add (self.passive_box, 0, wxALIGN_CENTER_HORIZONTAL)
		bsizer1.Add (5, 7)
		fsizer1 = wxFlexGridSizer (3, 3)
		fsizer1.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 10), (bsizer1, 1, wxEXPAND), (10, 10),
			(10, 10), (10, 10), (10, 10),
		])
		fsizer1.AddGrowableCol (1)
		fsizer1.AddGrowableRow (1)

		tidle_label = wxStaticText (self, -1, _("Thread Idle Timeout:"))
		self.tidle_entry = wxTextCtrl (self, -1)
		if opts and opts['thread_idle']:
			self.tidle_entry.SetValue (str (opts['thread_idle']))
		self.tidle_entry.SetSize ((100, -1))
		slabel4 = wxStaticText (self, -1, _("(seconds)"))
		hsizer = wxBoxSizer (wxHORIZONTAL)
		hsizer.Add (25, 1)
		hsizer.Add (tidle_label, 0, wxALIGN_CENTER_VERTICAL)
		hsizer.Add (10, 1, 1, wxEXPAND)
		hsizer.Add (self.tidle_entry, 0, wxEXPAND)
		hsizer.Add (10, 1, 1, wxEXPAND)
		hsizer.Add (slabel4, 0, wxALIGN_CENTER_VERTICAL)
		hsizer.Add (15, 1)
		box = wxStaticBox (self, -1, _("Thread Options"))
		bsizer2 = wxStaticBoxSizer (box, wxVERTICAL)
		bsizer2.Add (5, 7)
		bsizer2.Add (hsizer, 1, wxEXPAND)
		bsizer2.Add (5, 7)
		fsizer2 = wxFlexGridSizer (3, 3)
		fsizer2.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 10), (bsizer2, 1, wxEXPAND), (10, 10),
			(10, 10), (10, 10), (10, 10),
		])
		fsizer2.AddGrowableCol (1)
		fsizer2.AddGrowableRow (1)

		box = wxStaticBox (self, -1, _("Default Connection Settings"))
		boxsizer = wxStaticBoxSizer (box, wxVERTICAL)
		boxsizer.Add (fsizer1, 0, wxEXPAND)
		boxsizer.Add (fsizer2, 0, wxEXPAND)

		sizer = wxFlexGridSizer (3, 3)
		sizer.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 10), (boxsizer, 1, wxEXPAND), (10, 10),
			(10, 10), (10, 10), (10, 10),
		])
		sizer.AddGrowableCol (1)
		sizer.AddGrowableRow (1)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)
		sizer.Fit (self)
		sizer.SetSizeHints (self)

	def getOpts (self):
		opts = { }
		if self.ctimeout_entry.GetValue ():
			opts['timeout'] = int (self.ctimeout_entry.GetValue ())
		if self.idle_entry.GetValue ():
			opts['main_idle'] = int (self.idle_entry.GetValue ())
		if self.delay_entry.GetValue ():
			opts['delay'] = int (self.delay_entry.GetValue ())
		if self.attempts_entry.GetValue ():
			opts['retries'] = int (self.attempts_entry.GetValue ())
		if self.tidle_entry.GetValue ():
			opts['thread_idle'] = int (self.tidle_entry.GetValue ())
		opts['passive'] = bool (self.passive_box.GetValue ())
		return opts

class LoggingTab (wxPanel):

	idRADIO_ACTIVATE = wxNewId ()
	idRADIO_DEACTIVATE = wxNewId ()

	def __init__ (self, parent, opts=None):
		wxPanel.__init__ (self, parent, -1)

		self.activate_button = wxRadioButton (self, self.idRADIO_ACTIVATE,
		                                            _("Turn ON Logging"))
		self.deactivate_button = wxRadioButton (self, self.idRADIO_DEACTIVATE,
		                                              _("Turn OFF Logging"))
		if opts:
			if int (opts['logging']):
				self.activate_button.SetValue (true)
				self.log = True
			else:
				self.deactivate_button.SetValue (true)
				self.log = False

		EVT_RADIOBUTTON (self, self.idRADIO_ACTIVATE, self.onActivate)
		EVT_RADIOBUTTON (self, self.idRADIO_DEACTIVATE, self.onDeactivate)

		hsizer1 = wxBoxSizer (wxHORIZONTAL)
		hsizer1.Add (1, 1, 1, wxEXPAND)
		hsizer1.Add (self.activate_button, 0, wxEXPAND)
		hsizer1.Add (1, 1, 1, wxEXPAND)
		hsizer1.Add (self.deactivate_button, 0, wxEXPAND)
		hsizer1.Add (1, 1, 1, wxEXPAND)

		box = wxStaticBox (self, -1, _("Activate Logging"))
		bsizer1 = wxStaticBoxSizer (box, wxVERTICAL)
		bsizer1.Add (5, 7)
		bsizer1.Add (hsizer1, 1, wxEXPAND)
		bsizer1.Add (5, 7)
		fsizer1 = wxFlexGridSizer (3, 3)
		fsizer1.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 10), (bsizer1, 1, wxEXPAND), (10, 10),
			(10, 10), (10, 10), (10, 10),
		])
		fsizer1.AddGrowableCol (1)
		fsizer1.AddGrowableRow (1)

		cmd_label = wxStaticText (self, -1, _("FTP Command Log:"))
		download_label = wxStaticText (self, -1, _("Download Log:"))
		upload_label = wxStaticText (self, -1, _("Upload Log:"))
		self.cmd_entry = wxTextCtrl (self, -1)
		if opts and opts['cmd_log']:
			self.cmd_entry.SetValue (opts['cmd_log'])
		self.download_entry = wxTextCtrl (self, -1)
		if opts and opts['download_log']:
			self.download_entry.SetValue (opts['download_log'])
		self.upload_entry = wxTextCtrl (self, -1)
		if opts and opts['upload_log']:
			self.upload_entry.SetValue (opts['upload_log'])

		tsizer = wxGridSizer (3, 2)
		tsizer.AddMany ([
			(cmd_label, 0, wxALIGN_CENTER_VERTICAL), (self.cmd_entry, 1, wxEXPAND),
			(download_label, 0, wxALIGN_CENTER_VERTICAL), (self.download_entry, 1, wxEXPAND),
			(upload_label, 0, wxALIGN_CENTER_VERTICAL), (self.upload_entry, 1, wxEXPAND),
		])
		tsizer.SetVGap (7)
		fsizer2 = wxFlexGridSizer (3, 3)
		fsizer2.AddMany ([
			(20, 10), (10, 10), (20, 10),
			(20, 10), (tsizer, 1, wxEXPAND), (20, 10),
			(20, 10), (10, 10), (20, 10),
		])
		fsizer2.AddGrowableCol (1)
		fsizer2.AddGrowableRow (1)

		box = wxStaticBox (self, -1, _("Log Files"))
		bsizer2 = wxStaticBoxSizer (box, wxVERTICAL)
		bsizer2.Add (fsizer2, 1, wxEXPAND)
		fsizer3 = wxFlexGridSizer (3, 3)
		fsizer3.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 10), (bsizer2, 1, wxEXPAND), (10, 10),
			(10, 10), (10, 10), (10, 10),
		])
		fsizer3.AddGrowableCol (1)
		fsizer3.AddGrowableRow (1)

		box = wxStaticBox (self, -1, _("FTP Logging"))
		boxsizer = wxStaticBoxSizer (box, wxVERTICAL)
		boxsizer.Add (fsizer1, 0, wxEXPAND)
		boxsizer.Add (fsizer3, 0, wxEXPAND)

		sizer = wxFlexGridSizer (3, 3)
		sizer.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 10), (boxsizer, 1, wxEXPAND), (10, 10),
			(10, 10), (10, 10), (10, 10),
		])
		sizer.AddGrowableCol (1)
		sizer.AddGrowableRow (1)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)
		sizer.Fit (self)
		sizer.SetSizeHints (self)

	def onActivate (self, event):
		self.log = True
		self.activate_button.SetValue (true)

	def onDeactivate (self, event):
		self.log = False
		self.deactivate_button.SetValue (true)

	def getOpts (self):
		opts = { }
		opts['logging'] = self.log
		if self.cmd_entry.GetValue ():
			opts['cmd_log'] = self.cmd_entry.GetValue ()
		if self.download_entry.GetValue ():
			opts['download_log'] = self.download_entry.GetValue ()
		if self.upload_entry.GetValue ():
			opts['upload_log'] = self.upload_entry.GetValue ()
		return opts

class ColorsTab (wxPanel):

	idLOCAL_TEXT  = wxNewId ()
	idREMOTE_TEXT = wxNewId ()
	idERROR_TEXT  = wxNewId ()

	def __init__ (self, parent, opts=None):
		wxPanel.__init__ (self, parent, -1)

		self.color_chooser = wxPyColourChooser.wxPyColorChooser (self, -1)
		csizer = wxFlexGridSizer (3, 3)
		csizer.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 10), (self.color_chooser, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL), (10, 10),
			(10, 10), (10, 10), (10, 10),
		])

		self.local_button = wxRadioButton (self, self.idLOCAL_TEXT, _("Local Text"))
		self.remote_button = wxRadioButton (self, self.idREMOTE_TEXT, _("Remote Text"))
		self.error_button = wxRadioButton (self, self.idERROR_TEXT, _("Error Text"))
		EVT_RADIOBUTTON (self, self.idLOCAL_TEXT, self.onLocalClick)
		EVT_RADIOBUTTON (self, self.idREMOTE_TEXT, self.onRemoteClick)
		EVT_RADIOBUTTON (self, self.idERROR_TEXT, self.onErrorClick)

		hsizer = wxBoxSizer (wxHORIZONTAL)
		hsizer.Add (10, 1)
		hsizer.Add (self.local_button, 0, wxALIGN_CENTER_VERTICAL)
		hsizer.Add (15, 1)
		hsizer.Add (self.remote_button, 0, wxALIGN_CENTER_VERTICAL)
		hsizer.Add (15, 1)
		hsizer.Add (self.error_button, 0, wxALIGN_CENTER_VERTICAL)
		hsizer.Add (10, 1)
		box = wxStaticBox (self, -1, _("Text Type"))
		bsizer = wxStaticBoxSizer (box, wxVERTICAL)
		bsizer.Add (hsizer, 1, wxEXPAND)

		box = wxStaticBox (self, -1, _("Colors"))
		boxsizer = wxStaticBoxSizer (box, wxVERTICAL)
		boxsizer.Add (csizer, 0, wxEXPAND)
		boxsizer.Add (1, 5)
		boxsizer.Add (bsizer, 0, wxALIGN_CENTER_HORIZONTAL)
		sizer = wxFlexGridSizer (3, 3)
		sizer.AddMany ([
			(10, 10), (10, 10), (10, 10),
			(10, 10), (boxsizer, 1, wxEXPAND), (10, 10),
			(10, 10), (10, 10), (10, 10),
		])
		sizer.AddGrowableCol (1)
		sizer.AddGrowableRow (1)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)
		sizer.Fit (self)
		sizer.SetSizeHints (self)

		# Set current values and load the local text color as default
		self.local_color = (0, 0, 0)
		if opts and opts['local_color']:
			self.local_color = opts['local_color']
		self.remote_color = (0, 0, 0)
		if opts and opts['remote_color']:
			self.remote_color = opts['remote_color']
		self.error_color = (0, 0, 0)
		if opts and opts['error_color']:
			self.error_color = opts['error_color']
		self.local_button.SetValue (true)
		self.setColor (self.local_color)

	def setColor (self, color):
		if isinstance (color, wxColour):
			self.color_chooser.SetValue (color)
		elif isinstance (color, tuple):
			color = apply (wxColour, color)
			self.color_chooser.SetValue (color)

	def onLocalClick (self, event):
		if self.local_button.GetValue ():
			self.setColor (self.local_color)
		else:
			self.local_color = util.colorToTuple (self.color_chooser.GetValue ())
		self.local_button.SetValue (true)

	def onRemoteClick (self, event):
		if self.remote_button.GetValue ():
			self.setColor (self.remote_color)
		else:
			self.remote_color = util.colorToTuple (self.color_chooser.GetValue ())
		self.remote_button.SetValue (true)

	def onErrorClick (self, event):
		if self.error_button.GetValue ():
			self.setColor (self.error_color)
		else:
			self.error_color = util.colorToTuple (self.color_chooser.GetValue ())
		self.error_button.SetValue (true)

	def getOpts (self):
		# Update the colors to reflect any current changes
		if self.local_button.GetValue ():
			self.local_color = util.colorToTuple (self.color_chooser.GetValue ())
		elif self.remote_button.GetValue ():
			self.remote_color = util.colorToTuple (self.color_chooser.GetValue ())
		elif self.error_button.GetValue ():
			self.error_color = util.colorToTuple (self.color_chooser.GetValue ())

		opts = {
			'local_color'  : self.local_color,
			'remote_color' : self.remote_color,
			'error_color'  : self.error_color,
		}
		return opts
